#include "ScannerScore/ScannerSModels/C2HDM/ScannerSC2HDM.h"

using namespace std;


void C2HDM::UserInitCalcs(){
  //// Setting up the output print options
  //// leave the 1st argument at 1 if you want print points 1 by 1
  //// The 2nd argument indicates how many variables you want to print per point
  //setprint(1,89);
  setprint(1,89-65);
  STU_C2HDM.setup(nHzero,nHplus,nDoublet);

  #ifdef HBMODE
    //// Initialising HiggsBounds!
    HiggsBoundsAnalysis.InitialiseHiggsBounds();
  #ifdef HSMODE
    //// Initialising HiggsSignals!
    //int Np=0; //Number of free parameters of the model
    initialize_higgssignals_latestresults_(nHzero,nHplus);
    //setup_nparam_(Np); // Set number of extra free parameters for the model (no call defaults to 0)
    int pdf=2;	//choose probability density function of Higgs mass arround the measured central value (HiggsSignals)
    setup_pdf_(pdf);
  #endif
  #endif
  gsl_set_error_handler_off(); // handle possible errors in EDM integrals manually (by setting inf)


  cout<<"Type\tL1\tL2\tL3\tL4\tre_L5\tim_L5\tre_m12sq\ttbeta";
  cout<<"\tm_H1\tm_H2\tm_H3\tm_Hc\ta1\ta2\ta3";
  cout<<"\tm11sq\tm22sq\tim_m12sq";
  cout<<"\tS\tT\tU\tmaxev_uni\teEDM";
  cout<<"\tmu_F\tmu_V\tmu_gamgam\tmu_ZZ\tmu_WW\tmu_tautau\tmu_bb";//signal strenghts of 125 GeV Higgs
  cout<<"\tx_H1_ggF\tw_H1"; //H1 cxn and width
  cout<<"\tb_H1_bb\tb_H1_tautau\tb_H1_cc\tb_H1_ss\tb_H1_mumu\tb_H1_gg\tb_H1_gamgam\tb_H1_Zgam\tb_H1_WW\tb_H1_ZZ"; // H1 BRs
  cout<<"\tx_H2_ggF\tw_H2"; //H2 cxn and width
  cout<<"\tb_H2_tt\tb_H2_bb\tb_H2_tautau\tb_H2_cc\tb_H2_ss\tb_H2_mumu\tb_H2_gg\tb_H2_gamgam\tb_H2_Zgam\tb_H2_WW\tb_H2_ZZ\tb_H2_WpHm\tb_H2_ZH1\tb_H2_HpHm\tb_H2_H1H1"; // H2 BRs
  cout<<"\tx_H3_ggF\tw_H3"; //H3 cxn and width
  cout<<"\tb_H3_tt\tb_H3_bb\tb_H3_tautau\tb_H3_cc\tb_H3_ss\tb_H3_mumu\tb_H3_gg\tb_H3_gamgam\tb_H3_Zgam\tb_H3_WW\tb_H3_ZZ\tb_H3_WpHm\tb_H3_ZH1\tb_H3_ZH2\tb_H3_HpHm\tb_H3_H1H1\tb_H3_H2H2\tb_H3_H1H2"; // H3 BRs
  cout<<"\tw_Hc"; //H+ width
  cout<<"\tb_Hc_tb\tb_Hc_ts\tb_Hc_cb\tb_Hc_cs\tb_Hc_taunu\tb_Hc_WH1\tb_Hc_WH2\tb_Hc_WH3"; // H+ BRs
  cout<<endl;
  //// Use this vector to print/keep any extra data you calculate on UserAnalysis
  //// Keep it at 0 if you have no use for it
  extra_data.resize(0);
}

bool C2HDM::UserAnalysis(PhiRef & Phi,LambdaRef & L,MassRef & Mass, MmixingRef & Mixing){
  PhysicalStatesC2HDM();//lots of side effects
  /*if(not TestZbbarbLEP()){
    fail_Z_bbar++;
    return false;
  }*/
  /*if(abs(mH1-mH2)<30 || abs(mH2-mH3)<30 || mH1>125 || mH1>1000 || mH2>1000 || mH3>1000)
    return false;*/
  
  /* if(not Test_bs_gamma()){
    fail_bs_gamma++;
    return false;
  }

  if(not TestSTU()){
    fail_STU++;
    return false;
  }*/

  #ifdef HBMODE
  PrepareHiggsBoundsInput();

  int HBresult,chan,ncombined;
  double obsratio;
  HiggsBoundsAnalysis.run(HBresult,chan,obsratio,ncombined);
  if(HBresult!=1){
    fail_HB++;
    return false;
  }
  HB_channel=chan;

  if(!CheckSignalStrengths()){
    fail_rates++;
    return false;
  }

  if(not TestEDM()){
    fail_EDM++;
    return false;
  }

  #endif
  return true;
}

void C2HDM::what2print(){
  vector<double>::iterator print_it = points2print[store_vector_index].begin();
  *print_it++ = Type2HDM;
  *print_it++ = Cp[2];
  *print_it++ = Cp[3];
  *print_it++ = Cp[4];
  *print_it++ = Cp[5];
  *print_it++ = Cp[6];
  *print_it++ = Cp[7];
  *print_it++ = Cp[8];
  *print_it++ = tanbeta;

  *print_it++ = mH1;
  *print_it++ = mH2;
  *print_it++ = mH3;
  *print_it++ = mHc;
  *print_it++ = a1;
  *print_it++ = a2;
  *print_it++ = a3;

  *print_it++ = Cp[0];
  *print_it++ = Cp[1];
  *print_it++ = Cp[9];

  *print_it++ = STU_C2HDM.S;
  *print_it++ = STU_C2HDM.T;
  *print_it++ = STU_C2HDM.U;
  *print_it++ = maxev;
  *print_it++ = eEDM;

#ifdef HBMODE
  *print_it++ = mu_results["mu_F"];
  *print_it++ = mu_results["mu_V"];
  *print_it++ = mu_results["mu_gamgam"];
  *print_it++ = mu_results["mu_ZZ"];
  *print_it++ = mu_results["mu_WW"];
  *print_it++ = mu_results["mu_tautau"];
  *print_it++ = mu_results["mu_bb"];

//H1 properties
  *print_it++ = (InternalTableSusHiPP_ggF_noEW(13000,mH1,1,Rhf[0][0][0],Rhf[0][1][0])
    +InternalTableSusHiPP_ggF_noEW(13000,mH1,-1,Rhf[0][0][1],Rhf[0][1][1])
    +InternalTableSusHiPP_bbF_noEW(13000,mH1,1,Rhf[0][1][0])
    +InternalTableSusHiPP_bbF_noEW(13000,mH1,-1,Rhf[0][1][1]));
  *print_it++ = HiggsBoundsAnalysis.MhGammaTot[0];
  //branching ratios
  *print_it++ = HiggsBoundsAnalysis.BR_hjbb[0];
  *print_it++ = HiggsBoundsAnalysis.BR_hjtautau[0];
  *print_it++ = HiggsBoundsAnalysis.BR_hjcc[0];
  *print_it++ = HiggsBoundsAnalysis.BR_hjss[0];
  *print_it++ = HiggsBoundsAnalysis.BR_hjmumu[0];
  *print_it++ = HiggsBoundsAnalysis.BR_hjgg[0];
  *print_it++ = HiggsBoundsAnalysis.BR_hjgaga[0];
  *print_it++ = HiggsBoundsAnalysis.BR_hjZga[0];
  *print_it++ = HiggsBoundsAnalysis.BR_hjWW[0];
  *print_it++ = HiggsBoundsAnalysis.BR_hjZZ[0];

//H2 properties
  *print_it++ = (InternalTableSusHiPP_ggF_noEW(13000,mH2,1,Rhf[1][0][0],Rhf[1][1][0])
    +InternalTableSusHiPP_ggF_noEW(13000,mH2,-1,Rhf[1][0][1],Rhf[1][1][1])
    +InternalTableSusHiPP_bbF_noEW(13000,mH2,1,Rhf[1][1][0])
    +InternalTableSusHiPP_bbF_noEW(13000,mH2,-1,Rhf[1][1][1]));
  *print_it++ = HiggsBoundsAnalysis.MhGammaTot[1];
  //branching ratios
  *print_it++ = Hdecay.at("BR(H2 -> t tbar)");
  *print_it++ = HiggsBoundsAnalysis.BR_hjbb[1];
  *print_it++ = HiggsBoundsAnalysis.BR_hjtautau[1];
  *print_it++ = HiggsBoundsAnalysis.BR_hjcc[1];
  *print_it++ = HiggsBoundsAnalysis.BR_hjss[1];
  *print_it++ = HiggsBoundsAnalysis.BR_hjmumu[1];
  *print_it++ = HiggsBoundsAnalysis.BR_hjgg[1];
  *print_it++ = HiggsBoundsAnalysis.BR_hjgaga[1];
  *print_it++ = HiggsBoundsAnalysis.BR_hjZga[1];
  *print_it++ = HiggsBoundsAnalysis.BR_hjWW[1];
  *print_it++ = HiggsBoundsAnalysis.BR_hjZZ[1];
  *print_it++ = Hdecay.at("BR(H2 -> W+ H-)+BR(H2 -> W- H+)");
  *print_it++ = Hdecay.at("BR(H2 -> Z H1)");
  *print_it++ = Hdecay.at("BR(H2 -> H+ H-)");
  *print_it++ = HiggsBoundsAnalysis.BR_hjhihi[1][0]; // H1H1

//H3 properties
  *print_it++ = (InternalTableSusHiPP_ggF_noEW(13000,mH3,1,Rhf[2][0][0],Rhf[2][1][0])
    +InternalTableSusHiPP_ggF_noEW(13000,mH3,-1,Rhf[2][0][1],Rhf[2][1][1])
    +InternalTableSusHiPP_bbF_noEW(13000,mH3,1,Rhf[2][1][0])
    +InternalTableSusHiPP_bbF_noEW(13000,mH3,-1,Rhf[2][1][1]));
  *print_it++ = HiggsBoundsAnalysis.MhGammaTot[2];
  //branching ratios
  *print_it++ = Hdecay.at("BR(H3 -> t tbar)");
  *print_it++ = HiggsBoundsAnalysis.BR_hjbb[2];
  *print_it++ = HiggsBoundsAnalysis.BR_hjtautau[2];
  *print_it++ = HiggsBoundsAnalysis.BR_hjcc[2];
  *print_it++ = HiggsBoundsAnalysis.BR_hjss[2];
  *print_it++ = HiggsBoundsAnalysis.BR_hjmumu[2];
  *print_it++ = HiggsBoundsAnalysis.BR_hjgg[2];
  *print_it++ = HiggsBoundsAnalysis.BR_hjgaga[2];
  *print_it++ = HiggsBoundsAnalysis.BR_hjZga[2];
  *print_it++ = HiggsBoundsAnalysis.BR_hjWW[2];
  *print_it++ = HiggsBoundsAnalysis.BR_hjZZ[2];
  *print_it++ = Hdecay.at("BR(H3 -> W+ H-)+BR(H3 -> W- H+)");
  *print_it++ = Hdecay.at("BR(H3 -> Z H1)");
  *print_it++ = Hdecay.at("BR(H3 -> Z H2)");
  *print_it++ = Hdecay.at("BR(H3 -> H+ H-)");
  *print_it++ = HiggsBoundsAnalysis.BR_hjhihi[2][0]; // H1H1
  *print_it++ = HiggsBoundsAnalysis.BR_hjhihi[2][1]; // H2H2
  *print_it++ = Hdecay.at("BR(H3 -> H1 H2)");

//Charged Higgs - H^+
  *print_it++ = HiggsBoundsAnalysis.MHplusGammaTot[0];
  //branching ratios
  *print_it++ = Hdecay.at("BR(H+ -> t bbar)");
  *print_it++ = Hdecay.at("BR(H+ -> t sbar)");
  *print_it++ = HiggsBoundsAnalysis.BR_Hpjcb[0];
  *print_it++ = HiggsBoundsAnalysis.BR_Hpjcs[0];
  *print_it++ = HiggsBoundsAnalysis.BR_Hptaunu[0];
  *print_it++ = Hdecay.at("BR(H+ -> W+ H1)");
  *print_it++ = Hdecay.at("BR(H+ -> W+ H2)");
  *print_it++ = Hdecay.at("BR(H+ -> W+ H3)");
  
#endif

  if(print_it!=points2print[store_vector_index].end()) {
    cerr<<"ScannerS ERROR: Second argument of setprint does not match the attempted number of values to print!"<<endl;
    exit(EXIT_FAILURE);
  }
}

void C2HDM::UserFinalCalcs(void){
  //print failcounts
  clog<<"fail_global    "<<fail_global<<endl;
  clog<<"fail_Z_bbar    "<<fail_Z_bbar<<endl;
  clog<<"fail_bs_gamma  "<<fail_bs_gamma<<endl;
  clog<<"fail_STU       "<<fail_STU<<endl;
  clog<<"fail_EDM       "<<fail_EDM<<endl;
  clog<<"fail_HB        "<<fail_HB<<endl;
  clog<<"fail_rates     "<<fail_rates<<endl;

}

bool C2HDM::CheckStability(LambdaRef & L)const{
  ///////////////////////////////////////////
  ///////////// From 1507.05100 /////////////
  ///////////////////////////////////////////
  if(!(L[2]>0 && L[3]>0))
    return false;
  if(!(std::sqrt(L[2]*L[3])+L[4] > 0))
    return false;
  if(!(std::sqrt(L[2]*L[3])+L[4]+L[5] > std::sqrt(L[6]*L[6]+L[7]*L[7])))
    return false;

  return true;
}

bool C2HDM::CheckGlobal(PhiRef & Phi,LambdaRef & L,Potential & V)const{
  ///////////////////////////////////////////
  ///////////// From 1507.05100 /////////////
  ///////////////////////////////////////////
  double mHcsq_vsq = -(L[5]+L[6])/2+L[8]/(Phi[2]*Phi[6]);
  double disc = (pow(mHcsq_vsq +L[5]/2,2)-(L[6]*L[6]+L[7]*L[7])/4) * (mHcsq_vsq+(sqrt(L[2]*L[3])-L[4])/2);
  if(disc > 0)
    return true;
  else {
    fail_global++;
    return false;
  }
}


void C2HDM::MyPhiParametrization(const PhiParamVec & PhiPar,PhiVec & Phi){
//////////////////////////////////////////////////////////
//////// Define your VeV parameterization here ///////////
//////////////////////////////////////////////////////////

  // Example of the code used in the TwoHDM class

  Phi[2]=246/sqrt(1+PhiPar[0]*PhiPar[0]);
  Phi[6]=246*PhiPar[0]/sqrt(1+PhiPar[0]*PhiPar[0]);

}

void C2HDM::MyInternalMixing(const PhiParamVec & PhiPar,const PhiVec & Phi,MixingparamVec & MixPar,vector< vector<double> > & MixInternal){
//////////////////////////////////////
//////// unused by default ///////////
//////////////////////////////////////

  double a1=MixPar[0];
  double a2=1.*MixPar[1];
  double a3=1.*MixPar[2];
  double c1=cos(a1);
  double s1=sin(a1);
  double c2=cos(a2);
  double s2=sin(a2);
  double c3=cos(a3);
  double s3=sin(a3);
  
  /*MixInternal[0][0]=c1*c2;
  MixInternal[1][0]=s1*c2;
  MixInternal[2][0]=s2;  
  MixInternal[0][1]=-(c1*s2*s3+s1*c3);
  MixInternal[1][1]=c1*c3-s1*s2*s3;
  MixInternal[2][1]=c2*s3;  
  MixInternal[0][2]=-c1*s2*c3+s1*s3;
  MixInternal[1][2]=-(s1*s2*c3+c1*s3);
  MixInternal[2][2]=c2*c3;*/
  
  MixInternal[0][0]=c1*c2;
  MixInternal[0][1]=s1*c2;
  MixInternal[0][2]=s2;  
  MixInternal[1][0]=-(c1*s2*s3+s1*c3);
  MixInternal[1][1]=c1*c3-s1*s2*s3;
  MixInternal[1][2]=c2*s3;  
  MixInternal[2][0]=-c1*s2*c3+s1*s3;
  MixInternal[2][1]=-(s1*s2*c3+c1*s3);
  MixInternal[2][2]=c2*c3;
}

void C2HDM::MyCoupMassRelations(const PhiParamVec & PhiPar,const PhiVec & Phi,const vector< vector<double> > & Mixing,const MixingparamVec & MixPar,vector<double> & L,vector<double> & Mass, MLparamVec & MLPar, vector<double> & IndependentMs,vector<double> & IndependentLs){
//////////////////////////////////////////////////////////////////////////////////////
////// used to generate one neutral mass (chosen to be m2) in a box araound mHc //////
///// only active if the limits for the box >m2(mH+) are given in the input file /////
//////////////////////////////////////////////////////////////////////////////////////

IndependentMs.push_back(1);
Mass[1] = Mass[3] + MLPar[0];
if(Mass[1]>1000)
  Mass[1] = Mass[3]- MLPar[0];
}

bool C2HDM::UserCheckLocalMin(){
  //// User defined function to perform any extra checks on the local minimum that is generated in the first step
  
  return true;
}
