#include "ScannerScore/ScannerSModels/N2HDM/ScannerSN2HDM.h"

using namespace std;


//////////////////////////////////////////////////////////
////// Function contents to be defined by the user ///////
//////////////////////////////////////////////////////////
// WARNING: Don't change the prototypes of the template //
// functions  in this file since they are used by the   //
//     core routines. Only change their contents!       //
//////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////
//////////// General user defined function ///////////////
//////////////////////////////////////////////////////////
// These functions will be called when you use a        //
// Mathematica generated input file or if there is no   //
// Specific function for the model are using.           //
//////////////////////////////////////////////////////////

void N2HDM::UserInitCalcs(){
  //////////////////////////////////////////////////////////////////////////
  // ENTER HERE CODE FOR YOUR INITIAL CALCULATIONS BEFORE THE SCAN STARTS //
  //////////////////////////////////////////////////////////////////////////

  //// Setting up the output print options
  //// leave the 1st argument at 1 if you want print points 1 by 1
  //// The 2nd argument indicates how many variables you want to print per point
  setprint(1,133);

  STU_N2HDM.setup(nHzero,nHplus,nDoublet);

  #ifdef HBMODE
    //// Initialising HiggsBounds!
    HiggsBoundsAnalysis.InitialiseHiggsBounds();
  #ifdef HSMODE
    //// Initialising HiggsSignals!
    //int Np=0; //Number of free parameters of the model
    initialize_higgssignals_latestresults_(nHzero,nHplus);
    //setup_nparam_(Np); // Set number of extra free parameters for the model (no call defaults to 0)
    int pdf=2;	//choose probability density function of Higgs mass arround the measured central value (HiggsSignals)
    setup_pdf_(pdf);
  #endif
  #endif

  cout<<"p_mH1\tp_mH2\tp_mH3\tp_mA\tp_mHc\tp_tbeta\tp_a1\tp_a2\tp_a3\tp_m12sq\tp_vs\tp_THDMtype";//input parameters
  cout<<"\tp_L1\tp_L2\tp_L3\tp_L4\tp_L5\tp_L6\tp_L7\tp_L8\tp_u11sq\tp_u22sq\tp_ussq";//dependent parameters
  cout<<"\ts_sH1\ts_sH2\ts_sH3\ts_uni\ts_S\ts_T\ts_U\ts_HB_ch_id";//scan properties
  cout<<"\tmu_F\tmu_V\tmu_gamgam\tmu_ZZ\tmu_WW\tmu_tautau\tmu_bb";//signal strenghts of 125 GeV Higgs
  cout<<"\tx_H1_ggF\tw_H1"; //H1 cxn and width
  cout<<"\tb_H1_bb\tb_H1_tautau\tb_H1_cc\tb_H1_ss\tb_H1_mumu\tb_H1_gg\tb_H1_gamgam\tb_H1_Zgam\tb_H1_WW\tb_H1_ZZ\tb_H1_ZA\tb_H1_AA"; // H1 BRs
  cout<<"\tc_H1_top\tc_H1_bot\tc_H1_lep\tc_H1_VV\tc_H1_ZA"; // H1 couplings
  cout<<"\tx_H2_ggF\tw_H2"; //H2 cxn and width
  cout<<"\tb_H2_tt\tb_H2_bb\tb_H2_tautau\tb_H2_cc\tb_H2_ss\tb_H2_mumu\tb_H2_gg\tb_H2_gamgam\tb_H2_Zgam\tb_H2_WW\tb_H2_ZZ\tb_H2_ZA\tb_H2_WpHm\tb_H2_HpHm\tb_H2_AA\tb_H2_H1H1"; // H2 BRs
  cout<<"\tc_H2_top\tc_H2_bot\tc_H2_lep\tc_H2_VV\tc_H2_ZA"; // H2 couplings
  cout<<"\tx_H3_ggF\tw_H3"; //H3 cxn and width
  cout<<"\tb_H3_tt\tb_H3_bb\tb_H3_tautau\tb_H3_cc\tb_H3_ss\tb_H3_mumu\tb_H3_gg\tb_H3_gamgam\tb_H3_Zgam\tb_H3_WW\tb_H3_ZZ\tb_H3_ZA\tb_H3_WpHm\tb_H3_HpHm\tb_H3_AA\tb_H3_H1H1\tb_H3_H2H2\tb_H3_H1H2"; // H3 BRs
  cout<<"\tc_H3_top\tc_H3_bot\tc_H3_lep\tc_H3_VV\tc_H3_ZA"; // H3 couplings
  cout<<"\tx_A_ggF\tw_A"; //A cxn and width
  cout<<"\tb_A_tt\tb_A_bb\tb_A_tautau\tb_A_cc\tb_A_ss\tb_A_mumu\tb_A_gg\tb_A_gamgam\tb_A_Zgam\tb_A_WpHm\tb_A_ZH1\tb_A_ZH2\tb_A_ZH3"; // A BRs
  cout<<"\tc_A_top\tc_A_bot\tc_A_lep"; // A couplings
  cout<<"\tw_Hc"; //H+ width
  cout<<"\tb_Hc_tb\tb_Hc_ts\tb_Hc_cb\tb_Hc_cs\tb_Hc_taunu\tb_Hc_WA\tb_Hc_WH1\tb_Hc_WH2\tb_Hc_WH3"; // H+ BRs
  cout<<endl;
  //// Use this vector to print/keep any extra data you calculate on UserAnalysis
  //// Keep it at 0 if you have no use for it
  extra_data.resize(0);
}

bool N2HDM::UserAnalysis(PhiRef & Phi,LambdaRef & L,MassRef & Mass, MmixingRef & Mixing){
  ////////////////////////////////////////////////////////////////
  // ENTER CODE FOR YOUR TESTS/OUTPUT DURING THE SCAN 		//
  ////////////////////////////////////////////////////////////////
  // User defined analysis function to be executed once for 	//
  // each valid parameter space point that was generated. This	//
  // routine serves to accept or reject the point for example	//
  // through testing experimental constraints. 			//
  ////////////////////////////////////////////////////////////////
  PhysicalStatesN2HDM();//lots of side effects

  if(not TestZbbarbLEP()){
    fail_Z_bbar++;
    return false;
  }

  if(not Test_bs_gamma()){
    fail_bs_gamma++;
    return false;
  }

  if(not TestSTU()){
    fail_STU++;
    return false;
  }

  #ifdef HBMODE
  PrepareHiggsBoundsInput();

  int HBresult,chan,ncombined;
  double obsratio;
  HiggsBoundsAnalysis.run(HBresult,chan,obsratio,ncombined);
  if(HBresult!=1){
    fail_HB++;
    return false;
  }
  HB_channel=chan;

  if(!CheckSignalStrengths()){
    fail_rates++;
    return false;
  }

  #endif
  return true;
}

void N2HDM::UserFinalCalcs(void){
  //print failcounts
  clog<<"fail_global    "<<fail_global<<endl;
  clog<<"fail_Z_bbar    "<<fail_Z_bbar<<endl;
  clog<<"fail_bs_gamma  "<<fail_bs_gamma<<endl;
  clog<<"fail_STU       "<<fail_STU<<endl;
  clog<<"fail_HB        "<<fail_HB<<endl;
  clog<<"fail_rates     "<<fail_rates<<endl;
}


void N2HDM::what2print(){
  //////////////////////////////////////////////////////////////////////////
  //       CHOSE WHAT YOU WANT TO BE PRINTED TO THE OUTPUT FILE HERE      //
  //////////////////////////////////////////////////////////////////////////
  vector<double>::iterator print_it = points2print[store_vector_index].begin();
  // parameters in mass order
  *print_it++ = mH1;
  *print_it++ = mH2;
  *print_it++ = mH3;
  *print_it++ = mA;
  *print_it++ = mHc;
  *print_it++ = tanbeta;

  *print_it++ = a1;
  *print_it++ = a2;
  *print_it++ = a3;
  *print_it++ = Cp[11]; // m12^2
  *print_it++ = Phi[8]; // vs

  *print_it++ = Type2HDM;

  //dependent couplings
  *print_it++ = Cp[3]; // L1
  *print_it++ = Cp[4]; // L2
  *print_it++ = Cp[5]; // L3
  *print_it++ = Cp[6]; // L4
  *print_it++ = Cp[2]; // L5
  *print_it++ = Cp[8]; // L6
  *print_it++ = Cp[9]; // L7
  *print_it++ = Cp[10]; // L8
  *print_it++ = Cp[0]; // m11^2
  *print_it++ = Cp[1]; // m22^2
  *print_it++ = Cp[7]; // ms^2

  //scan properties
  *print_it++ = sH1; // singlet admixture of H1
  *print_it++ = sH2; // singlet admixture of H2
  *print_it++ = sH3; // singlet admixture of H3

  *print_it++ = maxev;//value in tree level unitarity test in multiples of Pi (<= 8
  // - STU
  *print_it++ = STU_N2HDM.S; // S parameter
  *print_it++ = STU_N2HDM.T; // T parameter
  *print_it++ = STU_N2HDM.U; // U parameter
  #ifdef HBMODE
  *print_it++ = HB_channel;
  // - signal strengths
  *print_it++ = mu_results["mu_F"];
  *print_it++ = mu_results["mu_V"];
  *print_it++ = mu_results["mu_gamgam"];
  *print_it++ = mu_results["mu_ZZ"];
  *print_it++ = mu_results["mu_WW"];
  *print_it++ = mu_results["mu_tautau"];
  *print_it++ = mu_results["mu_bb"];

  //H1 properties
    //production rate and total width
    *print_it++ = InternalTableSusHiPP_ggF_noEW(13000,mH1,1,Rhf[1][0],Rhf[1][1])+InternalTableSusHiPP_bbF_noEW(13000,mH1,1,Rhf[1][1]);
    *print_it++ = HiggsBoundsAnalysis.MhGammaTot[1];
    //branching ratios
    *print_it++ = HiggsBoundsAnalysis.BR_hjbb[1];
    *print_it++ = HiggsBoundsAnalysis.BR_hjtautau[1];
    *print_it++ = HiggsBoundsAnalysis.BR_hjcc[1];
    *print_it++ = HiggsBoundsAnalysis.BR_hjss[1];
    *print_it++ = HiggsBoundsAnalysis.BR_hjmumu[1];
    *print_it++ = HiggsBoundsAnalysis.BR_hjgg[1];
    *print_it++ = HiggsBoundsAnalysis.BR_hjgaga[1];
    *print_it++ = HiggsBoundsAnalysis.BR_hjZga[1];
    *print_it++ = HiggsBoundsAnalysis.BR_hjWW[1];
    *print_it++ = HiggsBoundsAnalysis.BR_hjZZ[1];
    *print_it++ = HdecayH1["BR(H1 -> Z A)"];
    *print_it++ = HiggsBoundsAnalysis.BR_hjhihi[1][3]; // AA
    //couplings
    *print_it++ = Rhf[1][0];// top
    *print_it++ = Rhf[1][1];// bot
    *print_it++ = Rhf[1][2];// lep
    *print_it++ = Rhvv[1];// VV
    *print_it++ = Rhhv[0][1]; // ZA



  //H2 properties
  //production rate and total width
    *print_it++ = InternalTableSusHiPP_ggF_noEW(13000,mH2,1,Rhf[2][0],Rhf[2][1])+InternalTableSusHiPP_bbF_noEW(13000,mH2,1,Rhf[2][1]);
    *print_it++ = HiggsBoundsAnalysis.MhGammaTot[2];
    //branching ratios
    *print_it++ = HdecayH2["BR(H2 -> t tbar)"];
    *print_it++ = HiggsBoundsAnalysis.BR_hjbb[2];
    *print_it++ = HiggsBoundsAnalysis.BR_hjtautau[2];
    *print_it++ = HiggsBoundsAnalysis.BR_hjcc[2];
    *print_it++ = HiggsBoundsAnalysis.BR_hjss[2];
    *print_it++ = HiggsBoundsAnalysis.BR_hjmumu[2];
    *print_it++ = HiggsBoundsAnalysis.BR_hjgg[2];
    *print_it++ = HiggsBoundsAnalysis.BR_hjgaga[2];
    *print_it++ = HiggsBoundsAnalysis.BR_hjZga[2];
    *print_it++ = HiggsBoundsAnalysis.BR_hjWW[2];
    *print_it++ = HiggsBoundsAnalysis.BR_hjZZ[2];
    *print_it++ = HdecayH2["BR(H2 -> Z A)"];
    *print_it++ = HdecayH2["BR(H2 -> W+ H-)+BR(H2 -> W- H+)"];
    *print_it++ = HdecayH2["BR(H2 -> H+ H-)"];
    *print_it++ = HiggsBoundsAnalysis.BR_hjhihi[2][0]; // AA
    *print_it++ = HiggsBoundsAnalysis.BR_hjhihi[2][1]; // H1H1
    //couplings
    *print_it++ = Rhf[2][0];// top
    *print_it++ = Rhf[2][1];// bot
    *print_it++ = Rhf[2][2];// lep
    *print_it++ = Rhvv[2];// VV
    *print_it++ = Rhhv[0][2]; // ZA


    //H3 properties
    //production rate and total width
      *print_it++ = InternalTableSusHiPP_ggF_noEW(13000,mH3,1,Rhf[3][0],Rhf[3][1])+InternalTableSusHiPP_bbF_noEW(13000,mH3,1,Rhf[3][1]);
      *print_it++ = HiggsBoundsAnalysis.MhGammaTot[3];
      //branching ratios
      *print_it++ = HdecayH3["BR(H3 -> t tbar)"];
      *print_it++ = HiggsBoundsAnalysis.BR_hjbb[3];
      *print_it++ = HiggsBoundsAnalysis.BR_hjtautau[3];
      *print_it++ = HiggsBoundsAnalysis.BR_hjcc[3];
      *print_it++ = HiggsBoundsAnalysis.BR_hjss[3];
      *print_it++ = HiggsBoundsAnalysis.BR_hjmumu[3];
      *print_it++ = HiggsBoundsAnalysis.BR_hjgg[3];
      *print_it++ = HiggsBoundsAnalysis.BR_hjgaga[3];
      *print_it++ = HiggsBoundsAnalysis.BR_hjZga[3];
      *print_it++ = HiggsBoundsAnalysis.BR_hjWW[3];
      *print_it++ = HiggsBoundsAnalysis.BR_hjZZ[3];
      *print_it++ = HdecayH3["BR(H3 -> Z A)"];
      *print_it++ = HdecayH3["BR(H3 -> W+ H-)+BR(H3 -> W- H+)"];
      *print_it++ = HdecayH3["BR(H3 -> H+ H-)"];
      *print_it++ = HiggsBoundsAnalysis.BR_hjhihi[3][0]; // AA
      *print_it++ = HiggsBoundsAnalysis.BR_hjhihi[3][1]; // H1H1
      *print_it++ = HiggsBoundsAnalysis.BR_hjhihi[3][2]; // H2H2
      *print_it++ = HdecayH3["BR(H3 -> H1 H2)"];
      //couplings
      *print_it++ = Rhf[3][0];// top
      *print_it++ = Rhf[3][1];// bot
      *print_it++ = Rhf[3][2];// lep
      *print_it++ = Rhvv[3];// VV
      *print_it++ = Rhhv[0][3]; // ZA

  //A properties
    //production rate and total width
    *print_it++ = InternalTableSusHiPP_ggF_noEW(13000,mA,-1,Rhf[0][0],Rhf[0][1])+InternalTableSusHiPP_bbF_noEW(13000,mA,-1,Rhf[0][1]);
    *print_it++ = HiggsBoundsAnalysis.MhGammaTot[0];
    //branching ratios
    *print_it++ = HdecayA["BR(A -> t tbar)"];
    *print_it++ = HiggsBoundsAnalysis.BR_hjbb[0];
    *print_it++ = HiggsBoundsAnalysis.BR_hjtautau[0];
    *print_it++ = HiggsBoundsAnalysis.BR_hjcc[0];
    *print_it++ = HiggsBoundsAnalysis.BR_hjss[0];
    *print_it++ = HiggsBoundsAnalysis.BR_hjmumu[0];
    *print_it++ = HiggsBoundsAnalysis.BR_hjgg[0];
    *print_it++ = HiggsBoundsAnalysis.BR_hjgaga[0];
    *print_it++ = HiggsBoundsAnalysis.BR_hjZga[0];
    *print_it++ = HdecayA["BR(A -> W+ H-)"];
    *print_it++ = HdecayA["BR(A -> Z H1)"];
    *print_it++ = HdecayA["BR(A -> Z H2)"];
    *print_it++ = HdecayA["BR(A -> Z H3)"];
    //couplings
    *print_it++ = Rhf[0][0];// top
    *print_it++ = Rhf[0][1];// bot
    *print_it++ = Rhf[0][2];// lep


  //Charged Higgs - H^+
    //total width
    *print_it++ = HiggsBoundsAnalysis.MHplusGammaTot[0];
    //branching ratios
    *print_it++ = HdecayHc["BR(H+ -> t bbar)"];
    *print_it++ = HdecayHc["BR(H+ -> t sbar)"];
    *print_it++ = HiggsBoundsAnalysis.BR_Hpjcb[0];
    *print_it++ = HiggsBoundsAnalysis.BR_Hpjcs[0];
    *print_it++ = HiggsBoundsAnalysis.BR_Hptaunu[0];
    *print_it++ = HdecayHc["BR(H+ -> W+ A)"];
    *print_it++ = HdecayHc["BR(H+ -> W+ H1)"];
    *print_it++ = HdecayHc["BR(H+ -> W+ H2)"];
    *print_it++ = HdecayHc["BR(H+ -> W+ H3)"];
  #endif
  if(print_it!=points2print[store_vector_index].end()) {
    cerr<<"ScannerS ERROR: Second argument of setprint does not match the attempted number of values to print!"<<endl;
    exit(EXIT_FAILURE);
  }
}

bool N2HDM::CheckStability(LambdaRef & L)const{
  ///////////////////////////////////////////
  ///// DOI: 10.1007/BF01034825 /////////////
  ///////////////////////////////////////////
  double sr12,sr16,sr26,discab,disc12,delta;

  if(!(L[8] > 0 && L[3] > 0 && L[4] > 0)) {
    // clog<<endl<<"---> ScannerS: Point rejected by L[8] > 0 && L[3] > 0 && L[4] > 0."<<endl;
    return false;
  }

  sr16=sqrt(L[3]*L[8]);
  sr26=sqrt(L[4]*L[8]);
  if(!(sr26+L[10]>0 && sr16+L[9]>0)) {
    // clog<<endl<<"---> ScannerS: Point rejected by sr26+L[10]>0 && sr16+L[9]>0."<<endl;
    return false;
  }

  sr12=sqrt(L[3]*L[4]);
  discab=L[9]+L[10]*sqrt(L[3]/L[4]);

  disc12 = L[6]-L[2];
  if(disc12>0) disc12=0;

  if(discab>=0 && disc12+sr12+L[5]>0){
    return true;
  }
  delta = sqrt((pow(L[9],2)-L[3]*L[8])*(pow(L[10],2)-L[4]*L[8]));
  if(discab<=0 && sr26>=L[10] && disc12*L[8]+delta >L[9]*L[10]){
    return true;
  }
  // clog<<endl<<"---> ScannerS: Point rejected by CheckStability()."<<endl;
  return false;
}

bool N2HDM::CheckGlobal(PhiRef & Phi0,LambdaRef & L0,Potential & V)const{
  ++fail_global;
  // variables
  double Vtest,v1_sq,vcp_sq,vcb_sq,vs_sq;

  //Bind references to auxiliary potential Vaux
  PhiRef Phi(V);
  LambdaRef L(V);

  //Copy values from current parameter space point
  Phi.copy(Phi0);
  L.copy(L0);

  // Compute potential at the current minimum
  double V0=V();


  // starting with the checks, analytic ones first
  if(L[11]<1e-3)return false; //not dealing with exact Z2

  // special case solutions where L[2]=L[6], both 2HDM and N2HDM case
  if(abs(L[2]-L[6])<1e-6){
    Vtest = ((pow(L[1],2)*L[3] + pow(L[0],2)*L[4] - 2*L[0]*L[1]*L[5])/
                (-(L[3]*L[4]) + pow(L[5],2)) - pow(L[11],2)/L[6])/2.;
    if(Vtest<V0) {
      // clog<<"special case 2HDM: "<<Vtest<<" < "<<V0<<endl;
      return false;
    }

    Vtest = (pow(L[5],2)*pow(L[7],2) - pow(L[0],2)*L[4]*L[8] +
          2*L[0]*L[1]*L[5]*L[8] + 2*L[0]*L[4]*L[7]*L[9] - 2*L[1]*L[5]*L[7]*L[9] +
          pow(L[1],2)*pow(L[9],2) - 2*L[0]*(L[5]*L[7] + L[1]*L[9])*L[10] +
          pow(L[0],2)*pow(L[10],2) -
          L[3]*(L[4]*pow(L[7],2) + L[1]*(L[1]*L[8] - 2*L[7]*L[10])))/
          (2*L[3]*L[4]*L[8] - 2*pow(L[5],2)*L[8] - 2*L[4]*pow(L[9],2) +
          4*L[5]*L[9]*L[10] - 2*L[3]*pow(L[10],2)) - pow(L[11],2)/(2.*L[6]);
    if(Vtest<V0) {
      // clog<<"special case N2HDM: "<<Vtest<<" < "<<V0<<endl;
      return false;
    }
  }

  // other analytic cases from simplest to most complicated

  // N2HDM - Case s
  if(2*L[7]/L[8]<0){//reality check
    Vtest = - pow(L[7],2)/2/L[8];
    if(Vtest<V0) {
      // clog<<"Case s: "<<Vtest<<" < "<<V0<<endl;
     return false;
    }
  }

  // 2HDM - Case IIb
  vcb_sq = (2*(L[1]*L[3] - L[0]*L[5]))/(-(L[3]*L[4]) + pow(L[5],2)) +
     (2*(-(L[3]*L[4]) + pow(L[5],2))*pow(L[11],2))/
      ((-(L[0]*L[4]) + L[1]*L[5])*pow(L[2] + L[6],2));
  v1_sq = (2*(L[0]*L[4] - L[1]*L[5]))/(-(L[3]*L[4]) + pow(L[5],2));
  if(vcb_sq > 0 && v1_sq > 0){
    Vtest = (pow(L[1],2)*L[3] + pow(L[0],2)*L[4] - 2*L[0]*L[1]*L[5])/
      (2.*(-(L[3]*L[4]) + pow(L[5],2))) - pow(L[11],2)/(L[2] + L[6]);
    if(Vtest<V0) {
    //  clog<<"Case IIb: "<<Vtest<<" < "<<V0<<endl;
     return false;
    }
  }

  //2HDM - Case IIa
  vcp_sq = (4*pow(L[2],2)*(L[1]*L[3] + L[0]*(L[2] - L[5] - L[6]))*
        (L[0]*L[4] + L[1]*(L[2] - L[5] - L[6])) -
       pow(-(L[3]*L[4]) + pow(-L[2] + L[5] + L[6],2),2)*pow(L[11],2))/
     (2.*pow(L[2],2)*(L[0]*L[4] + L[1]*(L[2] - L[5] - L[6]))*
       (-(L[3]*L[4]) + pow(-L[2] + L[5] + L[6],2)));
  v1_sq = (-2*L[0]*L[4] + 2*L[1]*(-L[2] + L[5] + L[6]))/
     (L[3]*L[4] - pow(-L[2] + L[5] + L[6],2));
  if(vcp_sq > 0 && v1_sq > 0){
    Vtest = (L[2]*(pow(L[1],2)*L[3] + pow(L[0],2)*L[4] +
          2*L[0]*L[1]*(L[2] - L[5] - L[6])) -
       (-(L[3]*L[4]) + pow(-L[2] + L[5] + L[6],2))*pow(L[11],2))/
     (2.*L[2]*(-(L[3]*L[4]) + pow(-L[2] + L[5] + L[6],2)));
    if(Vtest<V0){
      // clog<<"Case IIa: "<<Vtest<<" < "<<V0<<endl;
      return false;
    }
  }

  //N2HDM - Case IIb
  vcb_sq = (2*(-(L[1]*L[3]*L[8]) + L[0]*L[5]*L[8] - L[5]*L[7]*L[9] +
         L[1]*pow(L[9],2) + L[3]*L[7]*L[10] - L[0]*L[9]*L[10] +
         (pow(pow(L[5],2)*L[8] + L[4]*pow(L[9],2) - 2*L[5]*L[9]*L[10] +
              L[3]*(-(L[4]*L[8]) + pow(L[10],2)),2)*pow(L[11],2))/
          (pow(L[2] + L[6],2)*(-(L[1]*L[5]*L[8]) - L[4]*L[7]*L[9] +
              (L[5]*L[7] + L[1]*L[9])*L[10] + L[0]*(L[4]*L[8] - pow(L[10],2)))
            )))/(-(pow(L[5],2)*L[8]) - L[4]*pow(L[9],2) +
       2*L[5]*L[9]*L[10] + L[3]*(L[4]*L[8] - pow(L[10],2)));
  vs_sq = (2*(-(L[0]*L[4]*L[9]) + L[5]*(-(L[5]*L[7]) + L[1]*L[9] + L[0]*L[10]) +
         L[3]*(L[4]*L[7] - L[1]*L[10])))/
     (pow(L[5],2)*L[8] + L[4]*pow(L[9],2) - 2*L[5]*L[9]*L[10] +
       L[3]*(-(L[4]*L[8]) + pow(L[10],2)));
  v1_sq = (2*(-(L[1]*L[5]*L[8]) - L[4]*L[7]*L[9] + (L[5]*L[7] + L[1]*L[9])*L[10] +
         L[0]*(L[4]*L[8] - pow(L[10],2))))/
     (pow(L[5],2)*L[8] + L[4]*pow(L[9],2) - 2*L[5]*L[9]*L[10] +
       L[3]*(-(L[4]*L[8]) + pow(L[10],2)));
  if(vcb_sq > 0 && v1_sq > 0 && vs_sq > 0){ //reality check
    Vtest = -(L[2]*(pow(L[5],2)*pow(L[7],2) - pow(L[0],2)*L[4]*L[8] +
           2*L[0]*L[1]*L[5]*L[8] + 2*L[0]*L[4]*L[7]*L[9] -
           2*L[1]*L[5]*L[7]*L[9] + pow(L[1],2)*pow(L[9],2) -
           2*L[0]*(L[5]*L[7] + L[1]*L[9])*L[10] +
           pow(L[0],2)*pow(L[10],2) -
           L[3]*(L[4]*pow(L[7],2) + pow(L[1],2)*L[8] - 2*L[1]*L[7]*L[10]))\
         + L[6]*(pow(L[5],2)*pow(L[7],2) - pow(L[0],2)*L[4]*L[8] +
           2*L[0]*L[1]*L[5]*L[8] + 2*L[0]*L[4]*L[7]*L[9] -
           2*L[1]*L[5]*L[7]*L[9] + pow(L[1],2)*pow(L[9],2) -
           2*L[0]*(L[5]*L[7] + L[1]*L[9])*L[10] +
           pow(L[0],2)*pow(L[10],2) -
           L[3]*(L[4]*pow(L[7],2) + pow(L[1],2)*L[8] - 2*L[1]*L[7]*L[10]))\
         + 2*(pow(L[5],2)*L[8] + L[4]*pow(L[9],2) - 2*L[5]*L[9]*L[10] +
           L[3]*(-(L[4]*L[8]) + pow(L[10],2)))*pow(L[11],2))/
     (2.*(L[2] + L[6])*(pow(L[5],2)*L[8] + L[4]*pow(L[9],2) -
         2*L[5]*L[9]*L[10] + L[3]*(-(L[4]*L[8]) + pow(L[10],2))));
    if(Vtest<V0){
      // clog<<"Case sIIb: "<<Vtest<<" < "<<V0<<endl;
      return false;
    }
  }


  //N2HDM - Case IIa
  vcp_sq = (-4*L[1]*L[8] + 4*L[7]*L[10] + (4*
          (L[2]*L[8] - (L[5] + L[6])*L[8] + L[9]*L[10])*
          (L[7]*(-(L[4]*L[9]) + (-L[2] + L[5] + L[6])*L[10]) +
            L[1]*(L[2]*L[8] - (L[5] + L[6])*L[8] + L[9]*L[10]) +
            L[0]*(L[4]*L[8] - pow(L[10],2))))/
        (pow(L[2],2)*L[8] + L[4]*pow(L[9],2) +
          (L[5] + L[6])*((L[5] + L[6])*L[8] - 2*L[9]*L[10]) +
          2*L[2]*(-((L[5] + L[6])*L[8]) + L[9]*L[10]) +
          L[3]*(-(L[4]*L[8]) + pow(L[10],2))) -
       (L[4]*L[8]*(pow(L[2],2)*L[8] + L[4]*pow(L[9],2) +
            (L[5] + L[6])*((L[5] + L[6])*L[8] - 2*L[9]*L[10]) +
            2*L[2]*(-((L[5] + L[6])*L[8]) + L[9]*L[10]) +
            L[3]*(-(L[4]*L[8]) + pow(L[10],2)))*pow(L[11],2))/
        (pow(L[2],2)*(L[7]*(-(L[4]*L[9]) + (-L[2] + L[5] + L[6])*L[10]) +
            L[1]*(L[2]*L[8] - (L[5] + L[6])*L[8] + L[9]*L[10]) +
            L[0]*(L[4]*L[8] - pow(L[10],2)))) +
       (pow(L[10],2)*(pow(L[2],2)*L[8] + L[4]*pow(L[9],2) +
            (L[5] + L[6])*((L[5] + L[6])*L[8] - 2*L[9]*L[10]) +
            2*L[2]*(-((L[5] + L[6])*L[8]) + L[9]*L[10]) +
            L[3]*(-(L[4]*L[8]) + pow(L[10],2)))*pow(L[11],2))/
        (pow(L[2],2)*(L[7]*(-(L[4]*L[9]) + (-L[2] + L[5] + L[6])*L[10]) +
            L[1]*(L[2]*L[8] - (L[5] + L[6])*L[8] + L[9]*L[10]) +
            L[0]*(L[4]*L[8] - pow(L[10],2)))))/
     (2.*(L[4]*L[8] - pow(L[10],2)));
  vs_sq = (-2*(-(L[3]*L[4]) + pow(-L[2] + L[5] + L[6],2))*L[7] +
       2*(-(L[0]*L[4]) + L[1]*(-L[2] + L[5] + L[6]))*L[9] +
       2*(-(L[1]*L[3]) + L[0]*(-L[2] + L[5] + L[6]))*L[10])/
     (pow(L[2],2)*L[8] + L[4]*pow(L[9],2) +
       (L[5] + L[6])*((L[5] + L[6])*L[8] - 2*L[9]*L[10]) +
       2*L[2]*(-((L[5] + L[6])*L[8]) + L[9]*L[10]) +
       L[3]*(-(L[4]*L[8]) + pow(L[10],2)));
  v1_sq = (2*(L[7]*(-(L[4]*L[9]) + (-L[2] + L[5] + L[6])*L[10]) +
         L[1]*(L[2]*L[8] - (L[5] + L[6])*L[8] + L[9]*L[10]) +
         L[0]*(L[4]*L[8] - pow(L[10],2))))/
     (pow(L[2],2)*L[8] + L[4]*pow(L[9],2) +
       (L[5] + L[6])*((L[5] + L[6])*L[8] - 2*L[9]*L[10]) +
       2*L[2]*(-((L[5] + L[6])*L[8]) + L[9]*L[10]) +
       L[3]*(-(L[4]*L[8]) + pow(L[10],2)));
  if(vcp_sq > 0 && v1_sq > 0 && vs_sq > 0){ //reality check
    Vtest = (-(pow(L[2],3)*pow(L[7],2)) +
       (-(L[4]*pow(L[9],2)) + (L[5] + L[6])*
           (-((L[5] + L[6])*L[8]) + 2*L[9]*L[10]) +
          L[3]*(L[4]*L[8] - pow(L[10],2)))*pow(L[11],2) +
       pow(L[2],2)*(2*L[7]*((L[5] + L[6])*L[7] - L[1]*L[9] - L[0]*L[10]) +
          L[8]*(2*L[0]*L[1] - pow(L[11],2))) +
       L[2]*(-(pow(L[5],2)*pow(L[7],2)) - pow(L[6],2)*pow(L[7],2) +
          pow(L[0],2)*L[4]*L[8] - 2*L[0]*L[1]*L[6]*L[8] -
          2*L[0]*L[4]*L[7]*L[9] + 2*L[1]*L[6]*L[7]*L[9] -
          pow(L[1],2)*pow(L[9],2) + 2*L[0]*L[6]*L[7]*L[10] +
          2*L[0]*L[1]*L[9]*L[10] - pow(L[0],2)*pow(L[10],2) +
          L[3]*(L[4]*pow(L[7],2) + L[1]*(L[1]*L[8] - 2*L[7]*L[10])) +
          2*(L[6]*L[8] - L[9]*L[10])*pow(L[11],2) +
          2*L[5]*(L[7]*(-(L[6]*L[7]) + L[1]*L[9] + L[0]*L[10]) +
             L[8]*(-(L[0]*L[1]) + pow(L[11],2)))))/
     (2.*L[2]*(pow(L[2],2)*L[8] + L[4]*pow(L[9],2) +
         (L[5] + L[6])*((L[5] + L[6])*L[8] - 2*L[9]*L[10]) +
         2*L[2]*(-((L[5] + L[6])*L[8]) + L[9]*L[10]) +
         L[3]*(-(L[4]*L[8]) + pow(L[10],2))));
    if(Vtest<V0){
      // clog<<"Case sIIa: "<<Vtest<<" < "<<V0<<endl;
      return false;
    }
  }




  //numeric cases
  double * coeffs;
  coeffs = new double [4+1];
  double * s2b;
  s2b = new double [2*4];
  double vsq_temp[2];

  //2HDM - Case I
  Phi[0] = 0; //gauge inv
  Phi[1] = 0; //gauge inv
  Phi[3] = 0; //gauge inv
  Phi[4] = 0; //Case I
  Phi[5] = 0; //gauge inv
  Phi[7] = 0; //Case I
  Phi[8] = 0; //2HDM case

  coeffs[0]= pow(L[3],2)*pow(L[11],2);
  coeffs[1]= -pow(L[1]*L[3] - L[0]*(L[2] + L[5] + L[6]),2) -
     4*pow(L[3],2)*pow(L[11],2);
  coeffs[2]= (-(L[1]*L[3]) + L[0]*(L[2] + L[5] + L[6]))*
      (L[1]*(2*L[2] - 3*L[3] + 2*(L[5] + L[6])) +
        L[0]*(3*L[2] - 2*L[4] + 3*(L[5] + L[6]))) -
     2*L[3]*(-3*L[3] + L[4])*pow(L[11],2);
  coeffs[3]= -(pow(L[1],2)*(L[2] - 3*L[3] + L[5] + L[6])*(L[2] - L[3] + L[5] + L[6])) -
     pow(L[0],2)*(L[2] - L[4] + L[5] + L[6])*
      (3*L[2] - L[4] + 3*(L[5] + L[6])) -
     2*L[0]*L[1]*(2*pow(L[2],2) - L[2]*(3*L[3] + L[4] - 4*(L[5] + L[6])) +
        L[3]*(2*L[4] - 3*(L[5] + L[6])) - (L[5] + L[6])*(L[4] - 2*(L[5] + L[6]))
        ) + 4*L[3]*(-L[3] + L[4])*pow(L[11],2);
  coeffs[4]= pow(L[1]*(L[2] - L[3] + L[5] + L[6]) + L[0]*(L[2] - L[4] + L[5] + L[6]),
      2) + pow(L[3] - L[4],2)*pow(L[11],2);

  polyroots(coeffs,s2b,4);

  for(size_t i =0;i!=4;++i){ //iterate over four possible roots
    if(s2b[2*i+1]<1e-8*s2b[2*i] && s2b[2*i]< 1. && s2b[2*i] > 0.){ // check that root is real and sin^2(beta) ist in the range (0,1)
      vsq_temp[0] = (2*(L[1] + sqrt(-1 + 1/s2b[2*i])*L[11]))/
        ((-1 + s2b[2*i])*L[2] - L[5] - L[6] + s2b[2*i]*(-L[4] + L[5] + L[6]));
      vsq_temp[1] = (2*(L[1] - sqrt(-1 + 1/s2b[2*i])*L[11]))/
        ((-1 + s2b[2*i])*L[2] - L[5] - L[6] + s2b[2*i]*(-L[4] + L[5] + L[6]));
      for(size_t j = 0; j!=2; ++j){ //iterate over 2 posible signs of sin(beta)
        if(vsq_temp[j]>0) { // check that v^2 is positive <=> v is real
          // another 2 cases here: sign of v2 (assuming v1 is positive w.l.o.g.)
          // v2 > 0
          Phi[2]=sqrt((1-s2b[2*i])*vsq_temp[j]);
          Phi[6]=sqrt(s2b[2*i]*vsq_temp[j]); // v2 > 0
          Vtest = V();
          if(Vtest<V0) {
            // clog<<"Case I: "<<Vtest<<" < "<<V0<<endl;
            return false;
          }
          Phi[6]=-sqrt(s2b[2*i]*vsq_temp[j]); // v2 < 0
          Vtest = V();
          if(Vtest<V0) {
            // clog<<"Case I: "<<Vtest<<" < "<<V0<<endl;
            return false;
          }
        }
      }
    }
  }

  //N2HDM - Case I
  double vs_sq_2[2];

  Phi[0] = 0; //gauge inv
  Phi[1] = 0; //gauge inv
  Phi[3] = 0; //gauge inv
  Phi[4] = 0; //Case I
  Phi[5] = 0; //gauge inv
  Phi[7] = 0; //Case I

  coeffs[0]= pow(-(L[3]*L[8]) + pow(L[9],2),2)*pow(L[11],2);

  coeffs[1]= -pow(L[1]*(-(L[3]*L[8]) + pow(L[9],2)) +
    L[7]*(-((L[2] + L[5] + L[6])*L[9]) + L[3]*L[10]) +
    L[0]*((L[2] + L[5] + L[6])*L[8] - L[9]*L[10]),2) -
    4*pow(-(L[3]*L[8]) + pow(L[9],2),2)*pow(L[11],2);

  coeffs[2]= 3*pow(L[2],2)*pow(L[7],2)*pow(L[9],2) -
    2*L[2]*L[4]*pow(L[7],2)*pow(L[9],2) +
    6*L[2]*L[5]*pow(L[7],2)*pow(L[9],2) -
    2*L[4]*L[5]*pow(L[7],2)*pow(L[9],2) +
    3*pow(L[5],2)*pow(L[7],2)*pow(L[9],2) +
    6*L[2]*L[6]*pow(L[7],2)*pow(L[9],2) -
    2*L[4]*L[6]*pow(L[7],2)*pow(L[9],2) +
    6*L[5]*L[6]*pow(L[7],2)*pow(L[9],2) +
    3*pow(L[6],2)*pow(L[7],2)*pow(L[9],2) +
    2*pow(L[2],2)*pow(L[7],2)*L[9]*L[10] -
    6*L[2]*L[3]*pow(L[7],2)*L[9]*L[10] +
    2*L[3]*L[4]*pow(L[7],2)*L[9]*L[10] +
    4*L[2]*L[5]*pow(L[7],2)*L[9]*L[10] -
    6*L[3]*L[5]*pow(L[7],2)*L[9]*L[10] +
    2*pow(L[5],2)*pow(L[7],2)*L[9]*L[10] +
    4*L[2]*L[6]*pow(L[7],2)*L[9]*L[10] -
    6*L[3]*L[6]*pow(L[7],2)*L[9]*L[10] +
    4*L[5]*L[6]*pow(L[7],2)*L[9]*L[10] +
    2*pow(L[6],2)*pow(L[7],2)*L[9]*L[10] -
    2*L[2]*L[3]*pow(L[7],2)*pow(L[10],2) +
    3*pow(L[3],2)*pow(L[7],2)*pow(L[10],2) -
    2*L[3]*L[5]*pow(L[7],2)*pow(L[10],2) -
    2*L[3]*L[6]*pow(L[7],2)*pow(L[10],2) +
    pow(L[0],2)*((L[2] + L[5] + L[6])*L[8] - L[9]*L[10])*
    ((3*L[2] - 2*L[4] + 3*(L[5] + L[6]))*L[8] - 3*L[9]*L[10] +
    2*pow(L[10],2)) + 2*L[1]*L[7]*
    (-((pow(L[2],2) + pow(L[5] + L[6],2) +
    L[3]*(L[4] - 3*(L[5] + L[6])) + L[2]*(-3*L[3] + 2*(L[5] + L[6])))*
    L[8]*L[9]) + (-3*L[2] + L[4] - 3*(L[5] + L[6]))*pow(L[9],3) +
    L[3]*((2*L[2] - 3*L[3] + 2*(L[5] + L[6]))*L[8] + 3*pow(L[9],2))*
    L[10] - L[3]*L[9]*pow(L[10],2)) +
    pow(L[1],2)*(L[3]*L[8] - pow(L[9],2))*
    (-((2*L[2] - 3*L[3] + 2*(L[5] + L[6]))*L[8]) + L[9]*(-3*L[9] + 2*L[10]))
    + 2*L[0]*(L[1]*(L[8]*((pow(L[2],2) + pow(L[5] + L[6],2) +
    L[3]*(L[4] - 3*(L[5] + L[6])) +
    L[2]*(-3*L[3] + 2*(L[5] + L[6])))*L[8] +
    (3*L[2] - L[4] + 3*(L[5] + L[6]))*pow(L[9],2)) -
    L[9]*((2*L[2] - 3*L[3] + 2*(L[5] + L[6]))*L[8] + 3*pow(L[9],2))*
    L[10] + (-(L[3]*L[8]) + 2*pow(L[9],2))*pow(L[10],2)) +
    L[7]*(-((L[2] + L[5] + L[6])*(3*L[2] - 2*L[4] + 3*(L[5] + L[6]))*L[8]*
    L[9]) - ((pow(L[2],2) + pow(L[5] + L[6],2) +
    L[3]*(L[4] - 3*(L[5] + L[6])) +
    L[2]*(-3*L[3] + 2*(L[5] + L[6])))*L[8] +
    (-3*L[2] + L[4] - 3*(L[5] + L[6]))*pow(L[9],2))*L[10] -
    3*L[3]*L[9]*pow(L[10],2) + L[3]*pow(L[10],3))) +
    2*(L[3]*L[8] - pow(L[9],2))*
    (3*L[3]*L[8] - L[4]*L[8] - 3*pow(L[9],2) + pow(L[10],2))*
    pow(L[11],2);

  coeffs[3]= -3*pow(L[2],2)*pow(L[7],2)*pow(L[9],2) +
    4*L[2]*L[4]*pow(L[7],2)*pow(L[9],2) -
    pow(L[4],2)*pow(L[7],2)*pow(L[9],2) -
    6*L[2]*L[5]*pow(L[7],2)*pow(L[9],2) +
    4*L[4]*L[5]*pow(L[7],2)*pow(L[9],2) -
    3*pow(L[5],2)*pow(L[7],2)*pow(L[9],2) -
    6*L[2]*L[6]*pow(L[7],2)*pow(L[9],2) +
    4*L[4]*L[6]*pow(L[7],2)*pow(L[9],2) -
    6*L[5]*L[6]*pow(L[7],2)*pow(L[9],2) -
    3*pow(L[6],2)*pow(L[7],2)*pow(L[9],2) -
    pow(L[1],2)*((L[2] - L[3] + L[5] + L[6])*L[8] + L[9]*(L[9] - L[10]))*
    ((L[2] - 3*L[3] + L[5] + L[6])*L[8] + L[9]*(3*L[9] - L[10])) -
    4*pow(L[2],2)*pow(L[7],2)*L[9]*L[10] +
    6*L[2]*L[3]*pow(L[7],2)*L[9]*L[10] +
    2*L[2]*L[4]*pow(L[7],2)*L[9]*L[10] -
    4*L[3]*L[4]*pow(L[7],2)*L[9]*L[10] -
    8*L[2]*L[5]*pow(L[7],2)*L[9]*L[10] +
    6*L[3]*L[5]*pow(L[7],2)*L[9]*L[10] +
    2*L[4]*L[5]*pow(L[7],2)*L[9]*L[10] -
    4*pow(L[5],2)*pow(L[7],2)*L[9]*L[10] -
    8*L[2]*L[6]*pow(L[7],2)*L[9]*L[10] +
    6*L[3]*L[6]*pow(L[7],2)*L[9]*L[10] +
    2*L[4]*L[6]*pow(L[7],2)*L[9]*L[10] -
    8*L[5]*L[6]*pow(L[7],2)*L[9]*L[10] -
    4*pow(L[6],2)*pow(L[7],2)*L[9]*L[10] -
    pow(L[2],2)*pow(L[7],2)*pow(L[10],2) +
    4*L[2]*L[3]*pow(L[7],2)*pow(L[10],2) -
    3*pow(L[3],2)*pow(L[7],2)*pow(L[10],2) -
    2*L[2]*L[5]*pow(L[7],2)*pow(L[10],2) +
    4*L[3]*L[5]*pow(L[7],2)*pow(L[10],2) -
    pow(L[5],2)*pow(L[7],2)*pow(L[10],2) -
    2*L[2]*L[6]*pow(L[7],2)*pow(L[10],2) +
    4*L[3]*L[6]*pow(L[7],2)*pow(L[10],2) -
    2*L[5]*L[6]*pow(L[7],2)*pow(L[10],2) -
    pow(L[6],2)*pow(L[7],2)*pow(L[10],2) -
    pow(L[0],2)*((3*L[2] - L[4] + 3*(L[5] + L[6]))*L[8] - 3*L[9]*L[10] +
    pow(L[10],2))*((L[2] - L[4] + L[5] + L[6])*L[8] - L[9]*L[10] +
    pow(L[10],2)) + 2*L[1]*L[7]*
    ((2*pow(L[2],2) - L[2]*(3*L[3] + L[4] - 4*(L[5] + L[6])) +
    L[3]*(2*L[4] - 3*(L[5] + L[6])) -
    (L[5] + L[6])*(L[4] - 2*(L[5] + L[6])))*L[8]*L[9] +
    (3*L[2] - 2*L[4] + 3*(L[5] + L[6]))*pow(L[9],3) +
    ((L[2] - 3*L[3] + L[5] + L[6])*(L[2] - L[3] + L[5] + L[6])*L[8] +
    (-3*L[3] + L[4])*pow(L[9],2))*L[10] -
    (L[2] - 2*L[3] + L[5] + L[6])*L[9]*pow(L[10],2)) +
    2*L[0]*(L[7]*((L[2] - L[4] + L[5] + L[6])*
    (3*L[2] - L[4] + 3*(L[5] + L[6]))*L[8]*L[9] +
    ((2*pow(L[2],2) - L[2]*(3*L[3] + L[4] - 4*(L[5] + L[6])) +
    L[3]*(2*L[4] - 3*(L[5] + L[6])) -
    (L[5] + L[6])*(L[4] - 2*(L[5] + L[6])))*L[8] -
    (3*L[2] - 2*L[4] + 3*(L[5] + L[6]))*pow(L[9],2))*L[10] +
    (3*L[3] - L[4])*L[9]*pow(L[10],2) +
    (L[2] - 2*L[3] + L[5] + L[6])*pow(L[10],3)) +
    L[1]*(L[8]*((-2*pow(L[2],2) +
    L[2]*(3*L[3] + L[4] - 4*(L[5] + L[6])) +
    (L[5] + L[6])*(L[4] - 2*(L[5] + L[6])) +
    L[3]*(-2*L[4] + 3*(L[5] + L[6])))*L[8] -
    (3*L[2] - 2*L[4] + 3*(L[5] + L[6]))*pow(L[9],2)) +
    L[9]*((4*L[2] - 3*L[3] - L[4] + 4*(L[5] + L[6]))*L[8] +
    3*pow(L[9],2))*L[10] -
    ((L[2] - 2*L[3] + L[5] + L[6])*L[8] + 4*pow(L[9],2))*
    pow(L[10],2) + L[9]*pow(L[10],3))) -
    4*(-(L[3]*L[8]) + pow(L[9],2))*
    ((-L[3] + L[4])*L[8] + pow(L[9],2) - pow(L[10],2))*pow(L[11],2);

  coeffs[4]= pow(L[2],2)*pow(L[7],2)*pow(L[9],2) -
    2*L[2]*L[4]*pow(L[7],2)*pow(L[9],2) +
    pow(L[4],2)*pow(L[7],2)*pow(L[9],2) +
    2*L[2]*L[5]*pow(L[7],2)*pow(L[9],2) -
    2*L[4]*L[5]*pow(L[7],2)*pow(L[9],2) +
    pow(L[5],2)*pow(L[7],2)*pow(L[9],2) +
    2*L[2]*L[6]*pow(L[7],2)*pow(L[9],2) -
    2*L[4]*L[6]*pow(L[7],2)*pow(L[9],2) +
    2*L[5]*L[6]*pow(L[7],2)*pow(L[9],2) +
    pow(L[6],2)*pow(L[7],2)*pow(L[9],2) +
    pow(L[1],2)*pow((L[2] - L[3] + L[5] + L[6])*L[8] + L[9]*(L[9] - L[10]),
    2) + 2*pow(L[2],2)*pow(L[7],2)*L[9]*L[10] -
    2*L[2]*L[3]*pow(L[7],2)*L[9]*L[10] -
    2*L[2]*L[4]*pow(L[7],2)*L[9]*L[10] +
    2*L[3]*L[4]*pow(L[7],2)*L[9]*L[10] +
    4*L[2]*L[5]*pow(L[7],2)*L[9]*L[10] -
    2*L[3]*L[5]*pow(L[7],2)*L[9]*L[10] -
    2*L[4]*L[5]*pow(L[7],2)*L[9]*L[10] +
    2*pow(L[5],2)*pow(L[7],2)*L[9]*L[10] +
    4*L[2]*L[6]*pow(L[7],2)*L[9]*L[10] -
    2*L[3]*L[6]*pow(L[7],2)*L[9]*L[10] -
    2*L[4]*L[6]*pow(L[7],2)*L[9]*L[10] +
    4*L[5]*L[6]*pow(L[7],2)*L[9]*L[10] +
    2*pow(L[6],2)*pow(L[7],2)*L[9]*L[10] +
    pow(L[2],2)*pow(L[7],2)*pow(L[10],2) -
    2*L[2]*L[3]*pow(L[7],2)*pow(L[10],2) +
    pow(L[3],2)*pow(L[7],2)*pow(L[10],2) +
    2*L[2]*L[5]*pow(L[7],2)*pow(L[10],2) -
    2*L[3]*L[5]*pow(L[7],2)*pow(L[10],2) +
    pow(L[5],2)*pow(L[7],2)*pow(L[10],2) +
    2*L[2]*L[6]*pow(L[7],2)*pow(L[10],2) -
    2*L[3]*L[6]*pow(L[7],2)*pow(L[10],2) +
    2*L[5]*L[6]*pow(L[7],2)*pow(L[10],2) +
    pow(L[6],2)*pow(L[7],2)*pow(L[10],2) -
    2*L[1]*L[7]*((L[2] - L[3] + L[5] + L[6])*L[8] + L[9]*(L[9] - L[10]))*
    ((L[2] - L[4] + L[5] + L[6])*L[9] + (L[2] - L[3] + L[5] + L[6])*L[10]) +
    pow(L[0],2)*pow((L[2] - L[4] + L[5] + L[6])*L[8] - L[9]*L[10] +
    pow(L[10],2),2) + 2*L[0]*
    ((L[2] - L[4] + L[5] + L[6])*L[8] - L[9]*L[10] + pow(L[10],2))*
    (L[1]*((L[2] - L[3] + L[5] + L[6])*L[8] + L[9]*(L[9] - L[10])) +
    L[7]*(-((L[2] - L[4] + L[5] + L[6])*L[9]) -
    (L[2] - L[3] + L[5] + L[6])*L[10])) +
    pow((L[3] - L[4])*L[8] - pow(L[9],2) + pow(L[10],2),2)*pow(L[11],2);


  polyroots(coeffs,s2b,4);

  for(size_t i =0;i!=4;++i){ //iterate over four possible roots
    if(s2b[2*i+1]<1e-8*s2b[2*i] && s2b[2*i]< 1. && s2b[2*i] > 0.){ // check that root is real and sin^2(beta) ist in the range (0,1)
      vsq_temp[0]= (-2*L[7]*L[10] + 2*L[8]*(L[1] - sqrt(-1 + 1/s2b[2*i])*L[11]))/
        (((-1 + s2b[2*i])*L[2] - L[5] - L[6] + s2b[2*i]*(-L[4] + L[5] + L[6]))*L[8] -
        (-1 + s2b[2*i])*L[9]*L[10] + s2b[2*i]*pow(L[10],2));
      vs_sq_2[0]= (-2*((-1 + s2b[2*i])*L[2]*L[7] - (L[5] + L[6])*L[7] +
        L[9]*(L[1] - sqrt(-1 + 1/s2b[2*i])*L[11]) +
        s2b[2*i]*((-L[4] + L[5] + L[6])*L[7] -
        (L[9] - L[10])*(L[1] - sqrt(-1 + 1/s2b[2*i])*L[11]))))/
        (((-1 + s2b[2*i])*L[2] - L[5] - L[6] + s2b[2*i]*(-L[4] + L[5] + L[6]))*L[8] -
        (-1 + s2b[2*i])*L[9]*L[10] + s2b[2*i]*pow(L[10],2));

        vsq_temp[1]= (-2*L[7]*L[10] + 2*L[8]*(L[1] + sqrt(-1 + 1/s2b[2*i])*L[11]))/
          (((-1 + s2b[2*i])*L[2] - L[5] - L[6] + s2b[2*i]*(-L[4] + L[5] + L[6]))*L[8] -
          (-1 + s2b[2*i])*L[9]*L[10] + s2b[2*i]*pow(L[10],2));
        vs_sq_2[1]= (-2*((-1 + s2b[2*i])*L[2]*L[7] - (L[5] + L[6])*L[7] +
          L[9]*(L[1] + sqrt(-1 + 1/s2b[2*i])*L[11]) +
          s2b[2*i]*((-L[4] + L[5] + L[6])*L[7] -
          (L[9] - L[10])*(L[1] + sqrt(-1 + 1/s2b[2*i])*L[11]))))/
          (((-1 + s2b[2*i])*L[2] - L[5] - L[6] + s2b[2*i]*(-L[4] + L[5] + L[6]))*L[8] -
          (-1 + s2b[2*i])*L[9]*L[10] + s2b[2*i]*pow(L[10],2));

      for(size_t j = 0; j!=2; ++j){ //iterate over 2 posible signs of sin(beta) (as sign of the sqareroot in the eq above)
        if(vsq_temp[j]>0 && vs_sq_2[j]>0) { // check that v^2 is positive <=> v is real
            // another 2 cases here: sign of v2 (assuming w.l.o.g. that v1 and vs are positive)
            // v2 > 0
            Phi[2]=sqrt((1-s2b[2*i])*vsq_temp[j]);
            Phi[6]=sqrt(s2b[2*i]*vsq_temp[j]); // v2 > 0
            Phi[8]=sqrt(vs_sq_2[j]);
            Vtest = V();
            if(Vtest<V0 && fabs(pow(Phi[2],2)+pow(Phi[6],2)-pow(Phi0[2],2)-pow(Phi0[6],2))/pow(246.,2)>1e-6) {
              // clog<<"Case sI: "<<Vtest<<" < "<<V0<<endl;
              // clog<<"vsq: "<<sqrt(vsq_temp[j])<<" "<<sqrt(Phi0[2]*Phi0[2]+Phi0[6]*Phi0[6])<<endl;
              // clog<<"v1: "<<Phi[2]<<" "<<Phi0[2]<<endl;
              // clog<<"v2: "<<Phi[6]<<" "<<Phi0[6]<<endl;
              // clog<<"vs: "<<Phi[8]<<" "<<Phi0[8]<<endl;
              return false;
            }
            Phi[6]=-sqrt(s2b[2*i]*vsq_temp[j]); // v2 < 0
            Vtest = V();
            if(Vtest<V0 && fabs(pow(Phi[2],2)+pow(Phi[6],2)-pow(Phi0[2],2)-pow(Phi0[6],2))/pow(246.,2)>1e-6) {
              // clog<<"Case sI: "<<Vtest<<" < "<<V0<<endl;
              // clog<<"vsq: "<<sqrt(vsq_temp[j])<<" "<<sqrt(Phi0[2]*Phi0[2]+Phi0[6]*Phi0[6])<<endl;
              // clog<<"v1: "<<Phi[2]<<" "<<Phi0[2]<<endl;
              // clog<<"v2: "<<Phi[6]<<" "<<Phi0[6]<<endl;
              // clog<<"vs: "<<Phi[8]<<" "<<Phi0[8]<<endl;
              return false;
            }
        }
      }
    }
  }
  delete [] coeffs;
  delete [] s2b;
  --fail_global;
  return true;
}

//////////////////////////////////////////////////////////
//////// General user defined parameterizations //////////
//////////////////////////////////////////////////////////
// These functions will be called when you use a        //
// Mathematica generated input file.			//
//////////////////////////////////////////////////////////

void N2HDM::MyPhiParametrization(const PhiParamVec & PhiPar,PhiVec & Phi){
//////////////////////////////////////////////////////////
//////// Define your VeV parameterization here ///////////
//////////////////////////////////////////////////////////

  Phi[2]=246/sqrt(1+PhiPar[0]*PhiPar[0]);
  Phi[6]=246*PhiPar[0]/sqrt(1+PhiPar[0]*PhiPar[0]);
  Phi[8]=PhiPar[1];

}

void N2HDM::MyInternalMixing(const PhiParamVec & PhiPar,const PhiVec & Phi,MixingparamVec & MixPar,vector< vector<double> > & MixInternal){
/////////////////////////////////////////////////////////////
//////// Define your Mixing parameterization here ///////////
/////////////////////////////////////////////////////////////
  // //impose sin^2(delta)<tan^2(gamma)<cos^2(delta)
  // double alpha,gamma,delta,sign;
  // alpha = MixPar[0];
  // delta = MixPar[1];
  // sign = (MixPar[2] > 0) - (MixPar[2] < 0);
  // gamma = sign*atan(sin(abs(delta))+abs(MixPar[2])*(cos(delta)-sin(abs(delta))));
  // // Mixing Matrix
  // MixInternal[0][0] = cos(alpha)*cos(gamma);
  // MixInternal[0][1] = cos(gamma)*sin(alpha);
  // MixInternal[0][2] = sin(gamma);
  // MixInternal[1][0] = -(cos(delta)*sin(alpha)) - cos(alpha)*sin(delta)*sin(gamma);
  // MixInternal[1][1] = cos(alpha)*cos(delta) - sin(alpha)*sin(delta)*sin(gamma);
  // MixInternal[1][2] = cos(gamma)*sin(delta);
  // MixInternal[2][0] = sin(alpha)*sin(delta) - cos(alpha)*cos(delta)*sin(gamma);
  // MixInternal[2][1] = -(cos(alpha)*sin(delta)) - cos(delta)*sin(alpha)*sin(gamma);
  // MixInternal[2][2] = cos(delta)*cos(gamma);
}

void N2HDM::MyCoupMassRelations(const PhiParamVec & PhiPar,const PhiVec & Phi,const vector< vector<double> > & Mixing,const MixingparamVec & MixPar,vector<double> & L,vector<double> & Mass, MLparamVec & MLPar, vector<double> & IndependentMs,vector<double> & IndependentLs){
/////////////////////////////////////////////////////////////
////// Define your coupling/mass parameterization here //////
/////////////////////////////////////////////////////////////

}

bool N2HDM::UserCheckLocalMin(){
  //// User defined function to perform any extra checks on the local minimum that is generated in the first step
  
  return true;
}
