#include"ScannerSEWPO.h"

using namespace std;

////////////////////////
///// SM functions /////
////////////////////////

double DSsm(double Mass){
  double XZ2=pow(Mass,2)/Mz2;
  /*if(XZ2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DSsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  return Psi(XZ2)/Pi;
}

double DTsm(double Mass){
  double XW2=pow(Mass,2)/Mw2,XZ2=pow(Mass,2)/Mz2;
  /*if(XW2<1|| XZ2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DTsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
      }*/
  double term1,term2;
  if(abs(XW2-1)<1e-3)
    term1=(-3+XW2)/2e0;
  else
    term1=log(XW2)/(1-XW2);

  if(abs(XZ2-1)<1e-3)
    term2=(-3+XZ2)/2e0;
  else
    term2=log(XZ2)/(1-XZ2);

  return -3e0/16e0/Pi/sw2*XW2*(term1-term2);
}

double DUsm(double Mass){
  double XW2=pow(Mass,2)/Mw2,XZ2=pow(Mass,2)/Mz2;
  /*if(XW2<1|| XZ2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DUsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
      }*/

  return (Psi(XW2)-Psi(XZ2))/Pi;
}

double DVsm(double Mass){
  double XZ2=pow(Mass,2)/Mz2;
  /*if(XZ2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DVsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  return (Chi(XZ2)-Psi(XZ2))/4/Pi/sw2/cw2;
}

double DWsm(double Mass){
  double XW2=pow(Mass,2)/Mw2;
  /*if(XW2<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double DWsm(double Mass): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses Mass<M_V with Mass loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  return (Chi(XW2)-Psi(XW2))/4/Pi/sw2;
}

double DXsm(double Mass){
  return 0;
}

double Psi(double A){
  return Integrator(A,PsiIntegrand)+A*((4-A)*log(A)/(1-A)-0.5)/4e0;
}

double PsiIntegrand(double x,void * params){
  double A = *((double *) params);
  return (1/2e0+(1-A/2e0)*x-x*x/2e0)*log((1-x)*(1-x)+A*x);
}

double Chi(double A){
  return Integrator(A,ChiIntegrand);
}

double ChiIntegrand(double x,void * params){
  double A = *((double *) params);
  return x*(1-x)*(1e0/((1-x)*(1-x)+A*x)-0.5*log((1-x)*(1-x)+A*x));
}

///////////////// END OF SM functions ////////////////

//////////////////////////////////
////// N-singlet functions ///////
//////////////////////////////////

bool TestSTU_2sigma_xSM2_mixall(MassRef & Mass, MmixingRef & Mixing){
//by default we use 2 sigma
//this means, for 3 dof, chi2critical is 7.81
  return TestSTU_xSM2_mixall(Mass,Mixing,7.81);
}

bool TestSTU_xSM2_mixall(MassRef & Mass, MmixingRef & Mixing,double Chi2critical){

  vector<double> DOblique(3);
    //Set to central values
    DOblique[0]=STUgfitter[0];
    DOblique[1]=STUgfitter[1];
    DOblique[2]=STUgfitter[2];

    //Subtract model values
    DOblique[0]-=DS_nsinglets(Mass,Mixing,2)-DSsm(mHref);
    DOblique[1]-=DT_nsinglets(Mass,Mixing,2)-DTsm(mHref);
    DOblique[2]-=DU_nsinglets(Mass,Mixing,2)-DUsm(mHref);

    double chi2=0;
    for(vector<double>::size_type i=0; i!=3;++i){
      double sum=0;
      for(vector<double>::size_type j=0; j!=3;++j)
	sum+=InvCovSTUgfitter[i][j]*DOblique[j];//Compute Matrix*vector product
      chi2+=DOblique[i]*sum; //vector vector product
    }
    //If inside the 95% ellipsoid (for 3 dof chi2critical is 7.81)

    if(chi2<Chi2critical)
      return true;
    else
      return false;
}

double DS_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DSsm(Mass[a]);
  return result;
}

double DT_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DTsm(Mass[a]);
  return result;
}

double DU_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DUsm(Mass[a]);
  return result;
}

double DV_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DVsm(Mass[a]);
  return result;
}

double DW_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DWsm(Mass[a]);
  return result;
}

double DX_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih){
  double result=0e0;
  for(size_t a=0; a!=Mixing.Nmixed(); ++a)
    result+=pow(Mixing[a][ih],2)*DXsm(Mass[a]);
  return result;
}

///Alternative functions with vectors!!!

double DS_nsinglets2(std::vector<double> & Mass, std::vector<std::vector<double> > & Mixing,size_t ih){
  double result=0e0;  
  for(size_t a=0; a!=Mixing.size(); ++a)
    result+=pow(Mixing[a][ih],2)*DSsm(Mass[a]);
  return result;
}

double DT_nsinglets2(std::vector<double> & Mass, std::vector<std::vector<double> > & Mixing,size_t ih){
  double result=0e0;  
  for(size_t a=0; a!=Mixing.size(); ++a)
    result+=pow(Mixing[a][ih],2)*DTsm(Mass[a]);
  return result;
}

double DU_nsinglets2(std::vector<double> & Mass, std::vector<std::vector<double> > & Mixing,size_t ih){
  double result=0e0;  
  for(size_t a=0; a!=Mixing.size(); ++a)
    result+=pow(Mixing[a][ih],2)*DUsm(Mass[a]);
  return result;
}

////////// END OF n-singlets functions /////////


///////////////////////////
////// 2HDM routines //////
///////////////////////////

double DS_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta){
  return pow(cos(beta-alpha),2)*(DSsm(mHheavy)+F1(mA/Mz,mhlight/Mz)/2/Pi)+pow(sin(beta-alpha),2)*(DSsm(mhlight)+F1(mA/Mz,mHheavy/Mz)/2/Pi)+((2-4*cw2*sw2)*(F1(mHc/Mz,mHc/Mz)-log(mHc/Mz)/3e0)-F1(mHc/Mz,mHc/Mz))/2/Pi;
  //  return -(1/3e0*log(mHc*mHc)-pow(sin(beta-alpha),2)*Fdeltaline(mHheavy,mA)-pow(cos(beta-alpha),2)*Fdeltaline(mhlight,mA))/4/acos(-1);
}

double DT_2HDM(double mhlight, double mHheavy, double mA, double mHc,double alpha, double beta){
  return pow(cos(beta-alpha),2)*(DTsm(mHheavy)+(F0(mHc/Mz,mhlight/Mz)-F0(mA/Mz,mhlight/Mz))/16/Pi/sw2/cw2)+pow(sin(beta-alpha),2)*(DTsm(mhlight)+(F0(mHc/Mz,mHheavy/Mz)-F0(mA/Mz,mHheavy/Mz))/16/Pi/sw2/cw2)+F0(mHc/Mz,mA/Mz)/16/Pi/sw2/cw2;
  //double GF = 1.16639e-5;
  //double alphaem = 7.2973525698e-3;
  //return-(-F0(mA,mHc)+pow(sin(beta-alpha),2)*(F0(mHheavy,mA)-F0(mHheavy,mHc))+pow(cos(beta-alpha),2)*(F0(mhlight,mA)-F0(mhlight,mHc)))*sqrt(2)*GF/16e0/pow(acos(-1),2)/alphaem;
}

double DU_2HDM(double mhlight, double mHheavy, double mA, double mHc,double alpha, double beta){
  return pow(cos(beta-alpha),2)*(DUsm(mHheavy)+F1(mHc/Mw,mhlight/Mw)/2/Pi)+pow(sin(beta-alpha),2)*(DUsm(mhlight)+F1(mHc/Mw,mHheavy/Mw)/2/Pi)+(F1(mHc/Mw,mA/Mw)-2*log(mHc/Mw)/3)/2/Pi-DS_2HDM(mhlight,mHheavy,mA,mHc,alpha,beta)+pow(cos(beta-alpha),2)*DSsm(mHheavy)+pow(sin(beta-alpha),2)*DSsm(mhlight);
;
  //double GF = 1.16639e-5;
  //double alphaem = 7.2973525698e-3;
  //return-(-F0(mA,mHc)+pow(sin(beta-alpha),2)*(F0(mHheavy,mA)-F0(mHheavy,mHc))+pow(cos(beta-alpha),2)*(F0(mhlight,mA)-F0(mhlight,mHc)))*sqrt(2)*GF/16e0/pow(acos(-1),2)/alphaem;
}

double DV_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta){
  return pow(cos(beta-alpha),2)*(DVsm(mHheavy)+G1(mA/Mz,mhlight/Mz)/8/Pi/sw2/cw2)+pow(sin(beta-alpha),2)*(DVsm(mhlight)+G1(mA/Mz,mHheavy/Mz)/8/Pi/sw2/cw2)+G1(mHc/Mz,mHc/Mz)*(1-4*sw2*cw2)/8/Pi/sw2/cw2;
}

double DW_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta){
  return pow(cos(beta-alpha),2)*(DWsm(mHheavy)+G1(mHc/Mw,mhlight/Mw)/8/Pi/sw2)+pow(sin(beta-alpha),2)*(DWsm(mhlight)+G1(mHc/Mw,mHheavy/Mw)/8/Pi/sw2)+G1(mHc/Mw,mA/Mw)/8/Pi/sw2;
}

double DX_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta){
  return pow(cos(beta-alpha),2)*DXsm(mHheavy)+pow(sin(beta-alpha),2)*DXsm(mhlight)+(cw2-sw2)*G2func(mHc*mHc/Mz2)/4/Pi;
}

double F0(double A, double B){//F0=(A^2+B^2)/2-A^2B^2Log(A^2/B^2)/(A^2-B^2)
  double SumSq=A*A+B*B,delta=(A*A-B*B)/SumSq;//Auxiliary variables which are useful to test if close to numerically difficult point when masses are degenerate
  if (abs(delta)<1e-3)
    return abs(delta)+SumSq*delta*delta*(1+delta*delta/5e0)/3e0; //Use series expansion if close to such point...
  else
    return SumSq*(0.5+0.25*(delta-1)*(delta+1)/delta*log((1+delta)/(1-delta)));//Otherwise use full expression (simplified here in terms of delta and SumSq)
}

double F1(double A, double B){
  /*if(A+B<1){
    cerr<<"ScannerS error in ScannerSEWPO.cpp: double F1(double A, double B): Loop functions for 2HDM called with masses such that corrections are not virtual (i.e. masses A+B<M_V with A,B loop particle masses and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  double * Msq;
  Msq= new double [2];
  Msq[0]=A*A;
  Msq[1]=B*B;
  double result=IntegratorDx(IntegrandF1,Msq)-1e0/6e0;
  delete Msq;
  return result;
}

double IntegrandF1(double x,void * params){
  double * Msq = (double *) params;
  double Prod=x*(1-x),ArgLog=(1-x)*Msq[0]+x*Msq[1]-Prod;
  //std::clog<<"msq0="<<Msq[0]<< " Msq[1]="<<Msq[1]<< endl;
  //return Prod*log(ArgLog)-(ArgLog+Prod)*log(ArgLog/(ArgLog+Prod));
  return (ArgLog+Prod)*log(ArgLog+Prod)-ArgLog*log(ArgLog);
}

double G1(double A, double B){
  /*if(A+B<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double G1(double A, double B): Loop functions for 2HDM called with masses such that corrections are not virtual (i.e. masses A+B<M_V with A,B loop particle masses and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  double * Msq;
  Msq= new double [2];
  Msq[0]=A*A;
  Msq[1]=B*B;
  double result=IntegratorDx(IntegrandG1,Msq)+1e0/6e0;
  delete Msq;
  return result;
}

double IntegrandG1(double x,void * params){
  double * Msq = (double *) params;
  double Prod=x*(1-x),ArgLog=(1-x)*Msq[0]+x*Msq[1]-Prod;
  return (ArgLog+Prod)*log(ArgLog/(ArgLog+Prod));
}

double G2func(double A){
  /*if(A<1){
    cerr<<"ScannerS error in ScannerSEWPO.h: double G2func(double A): Loop function for 2HDM called with mass such that corrections are not virtual (i.e. masses A<M_V with A loop particle mass and M_V  vector boson mass). Check your code to prevent this from happening!" << endl;
      exit(EXIT_FAILURE);
  }*/
  return Integrator(A,IntegrandG2func)+1e0/6e0;
}

double IntegrandG2func(double x,void * params){
  double A = *((double *) params);
  return (A*A-x*(1-x))*log(1-x*(1-x)/A/A);
}

bool CheckVirtual(double m1, double m2){
  double m1sq_Mw2=pow(m1,2)/Mw2,m1sq_Mz2=pow(m1,2)/Mz2,m2sq_Mw2=pow(m2,2)/Mw2,m2sq_Mz2=pow(m2,2)/Mz2;
  double Sw2=m1sq_Mw2+m2sq_Mw2,Sz2=m1sq_Mz2+m2sq_Mz2;
  double dw2=m1sq_Mw2-m2sq_Mw2,dz2=m1sq_Mz2-m2sq_Mz2;
  if((abs(dw2)<1 && Sw2<=(dw2*dw2+1)/2e0)||(abs(dz2)<1 && Sz2<=(dz2*dz2+1)/2e0))
    return false;
  if(Sw2<=abs(dw2)||Sz2<=abs(dz2))
    return false;
  return true;
}

bool TestSTU_2sigma_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta){
//by default we use 2 sigma
//this means, for 3 dof, chi2critical is 7.81
  return TestSTU_2HDM(mhlight,mHheavy,mA,mHc,alpha,beta,7.81);
}

bool TestSTU_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta,double Chi2critical){

  if(CheckVirtual(mA,mHc)&& CheckVirtual(mA,mhlight) && CheckVirtual(mA,mHheavy) && CheckVirtual(mHc,mHc) && CheckVirtual(mHc,mhlight) && CheckVirtual(mHc,mHheavy) && mHc>=50){
    vector<double> DOblique(3);
    //Set to central values
    DOblique[0]=STUgfitter[0];
    DOblique[1]=STUgfitter[1];
    DOblique[2]=STUgfitter[2];

    //Subtract model values
    DOblique[0]-=DS_2HDM(mhlight,mHheavy,mA,mHc,alpha,beta)-DSsm(mHref);
    DOblique[1]-=DT_2HDM(mhlight,mHheavy,mA,mHc,alpha,beta)-DTsm(mHref);
    DOblique[2]-=DU_2HDM(mhlight,mHheavy,mA,mHc,alpha,beta)-DUsm(mHref);

    double chi2=0;
    for(vector<double>::size_type i=0; i!=3;++i){
      double sum=0;
      for(vector<double>::size_type j=0; j!=3;++j)
	sum+=InvCovSTUgfitter[i][j]*DOblique[j];//Compute Matrix*vector product
      chi2+=DOblique[i]*sum; //vector vector product
    }
    //If inside the 95% ellipsoid (for 3 dof chi2critical is 7.81)
    if(chi2<Chi2critical)
      return true;
    else
      return false;
  }
  else{
    std::clog<<"Warning in bool TestSTU_2sigma_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta): Attempt to aplly STU test for too low masses. Test ignored for this point."<<endl;
    return true;
  }

}


//-------------------------------------------------------------
//---------------------- STUchecker ---------------------------
//- checks STU in models with arbitraty doublets and singlets -
//------ the calculation follows Grimus et al. 0802.4353 ------
//-------------------------------------------------------------

// routine to allocate the matrices needed for the STU calculation
// input: nDoublet = Number of doublets in the Theory
//        nHplus = Number of massive charged states
//        nHzero = Number of massive neutral states
void STUchecker::setup(size_t nHzero, size_t nHplus, size_t nDoublet) {
  // input variables
  mHzero.resize(nHzero);
  mHplus.resize(nHplus);

  m_U.resize(nDoublet);
  m_Vr.resize(nDoublet);
  m_Vi.resize(nDoublet);
  for(size_t i=0; i != nDoublet; ++i){
    m_U.at(i).resize(nHplus+1);
    m_Vr.at(i).resize(nHzero + 1);
    m_Vi.at(i).resize(nHzero + 1);
  }


  // internal mass vectors including the goldstone
  mcsq.resize(nHplus + 1);
  m0sq.resize(nHzero + 1);
  mcsq[0]=0;
  m0sq[0]=0;
  // some shorthands
  m = m0sq.size();
  n = mcsq.size();
}

// before using this routine you need to supply values
// to the global variables of the STUchecker object
// input: mHzero = masses of the neutral states
//        mHplus = masses of the charged states
//        U = charged sector mixing matrix U as defined in 0802.4353
//        Vr,Vi = real/imaginary part of the neutral sector mixing matrix V as in 0802.4353
bool STUchecker::evaluate() {
  // transfer and square masses
  for(size_t i = 1;i != m; ++i) m0sq[i]=pow(mHzero[i-1],2);
  for(size_t i = 1;i != n; ++i) mcsq[i]=pow(mHplus[i-1],2);

  vector<double> DOblique(3);
  constexpr double Chi2critical = 7.81;
  //Set to central values
  DOblique[0]=STUgfitter[0];
  DOblique[1]=STUgfitter[1];
  DOblique[2]=STUgfitter[2];

  // clog<<"ImVVsq={{";
  // for(size_t b1 = 0;b1 <  m_Vr[0].size();++b1){
  //   for(size_t b2 = 0;b2 <  m_Vr[0].size();++b2){
  //     clog<<ImVVsq(b1,b2)<<",";
  //   }
  //   clog<<"\b},{";
  // }
  // clog<<"\b\b\b}}\n";
  //
  // clog<<"UVsq={{";
  // for(size_t a = 0;a <  m_U[0].size();++a){
  //   for(size_t b = 0;b <  m_Vr[0].size();++b){
  //     clog<<UVsq(a,b)<<",";
  //   }
  //   clog<<"\b},{";
  // }
  // clog<<"\b\b\b}}\n";
  //
  // clog<<"UUsq={{";
  // for(size_t a1 = 0;a1 <  m_U[0].size();++a1){
  //   for(size_t a2 = 0;a2 <  m_U[0].size();++a2){
  //     clog<<UUsq(a1,a2)<<",";
  //   }
  //   clog<<"\b},{";
  // }
  // clog<<"\b\b\b}}\n";
  //
  // clog<<"dUU={";
  // for(size_t a = 0;a <  m_U[0].size();++a){
  //   clog<<dUU(a)<<",";
  // }
  // clog<<"\b}\n";
  //
  // clog<<"dVV={";
  // for(size_t b = 0;b <  m_Vr[0].size();++b){
  //   clog<<dVV(b)<<",";
  // }
  // clog<<"\b}\n";

  S = calc_S();
  T = calc_T();
  U = calc_U();

  //Subtract model values
  DOblique[0]-=S;
  DOblique[1]-=T;
  DOblique[2]-=U;

  double chi2=0;
  for(vector<double>::size_type i=0; i!=3;++i){
    double sum=0;
    for(vector<double>::size_type j=0; j!=3;++j)
  sum+=InvCovSTUgfitter[i][j]*DOblique[j];//Compute Matrix*vector product
    chi2+=DOblique[i]*sum; //vector vector product
  }
  //If inside the 95% ellipsoid (for 3 dof chi2critical is 7.81)
  if(chi2<Chi2critical)
    return true;
  else
    return false;

  return true;
}


double STUchecker::ImVVsq(size_t b1,size_t b2){
  double temp = 0;
  for(size_t k = 0; k!=m_Vr.size(); k++){
    temp+=m_Vr[k][b1]*m_Vi[k][b2]-m_Vi[k][b1]*m_Vr[k][b2];
  }
  return pow(temp,2);
}

double STUchecker::UVsq(size_t a, size_t b){
  double tempr = 0;
  double tempi = 0;
  for(size_t k = 0; k!=m_U.size(); k++){
    tempr+=m_U[k][a]*m_Vr[k][b];
    tempi+=m_U[k][a]*m_Vi[k][b];
  }
  return pow(tempr,2)+pow(tempi,2);
}

double STUchecker::UUsq(size_t a1, size_t a2){
  double temp = 0;
  for(size_t k = 0; k!=m_U.size(); k++){
    temp+=m_U[k][a1]*m_U[k][a2];
  }
  return pow(temp,2);
}

double STUchecker::dUU(size_t a){
  double temp = 0;
  for(size_t k = 0; k!=m_U.size(); k++){
    temp+=pow(m_U[k][a],2);
  }
  return temp;
}

double STUchecker::dVV(size_t b){
  double temp = 0;
  for(size_t k = 0; k!=m_Vr.size(); ++k){
    temp+=pow(m_Vr[k][b],2)+pow(m_Vi[k][b],2);
  }
  return temp;
}

// all eq. numbers refer to 0802.4353

// eq. 29
// expanded to make numerics safe
double STUchecker::F(double I, double J){ // F=(I+J)/2-I*J*Log(I/J)/(I-J)
  double SumSq=I+J,delta=(I-J)/SumSq;
  if (abs(delta)<1e-3)
    return abs(delta)+SumSq*delta*delta*(1+delta*delta/5e0)/3e0;
  else
    return SumSq*(0.5+0.25*(delta-1)*(delta+1)/delta*log((1+delta)/(1-delta)));
}

// eq. B7
// all poles in the region t>0 are taken care of
double STUchecker::func_f(double t,double r) {
  if (abs(r)<1e-3)
    return -2*r/t;
  double sqr;
  if(r > 0) {
    sqr = sqrt(r);
    return sqr*log(abs((t-sqr)/(t+sqr)));
  }
  else {
    sqr = sqrt(-r);
    return 2*sqr*atan(sqr/t);
  }
}

// eq. C2
// all poles in the region I,J,Q>0 are taken care of
double STUchecker::G(double I, double J, double Q) {
  double logpart,r,t;
  if(abs(I-J)<1e-3)
    logpart = 6.0*J/Q+3.0*(I-J)/Q;
  else
    logpart = 3.0/Q*((pow(I,2)+pow(J,2))/(I-J)-(pow(I,2)-pow(J,2))/Q+pow(I-J,3)/(3.0*pow(Q,2)))*log(I/J);
  r = pow(Q,2)-2*Q*(I+J)+pow(I-J,2);
  t = I+J-Q;
  return -16.0/3.0+5*(I+J)/Q-2*pow(I-J,2)/pow(Q,2)+logpart+r/pow(Q,3)*func_f(t,r);
}

// eq. C5
// all poles in the region I,Q>0 are taken care of
double STUchecker::G2(double I,double Q) {
  double logpart;
  if(abs(I-Q)<1e-3)
    logpart = -18.0+3.0*(I-Q)/Q;
  else
    logpart = (-10.0+18.0*I/Q-6*pow(I/Q,2)+pow(I/Q,3)-9*(I+Q)/(I-Q))*log(I/Q);
  return -79./3.+9.*I/Q-2.*pow(I/Q,2)+logpart+(12-4*I/Q+pow(I/Q,2))*func_f(I,pow(I,2)-4.*I*Q)/Q;
}

// eq. 28
// line by line with prefactors from eq. 8 incl.
double STUchecker::calc_T() {
  using exp_c::pi;
  double T0=0;
  for(size_t a = 1; a < n; ++a){
    for(size_t b = 1; b < m; ++b){
      T0 += UVsq(a,b)*F(mcsq[a],m0sq[b]);
    }
  }
  for(size_t b1 = 1; b1 < m - 1; ++b1){
    for(size_t b2 = b1 + 1; b2 < m; ++b2){
      T0 -= ImVVsq(b1,b2)*F(m0sq[b1],m0sq[b2]);
    }
  }
  for(size_t a1 = 1; a1 < n - 1; ++a1){
    for(size_t a2 = a1 + 1; a2 < n; ++a2){
      T0 -= 2*UUsq(a1,a2)*F(mcsq[a1],mcsq[a2]);
    }
  }
  for(size_t b = 1; b < m; ++b){
    T0 += 3*ImVVsq(0,b)*(F(mz2,m0sq[b])-F(mw2,m0sq[b]));
  }
  T0 -= 3*(F(mz2,mhref2)-F(mw2,mhref2));
  return T0/16/Pi/sw2/mw2;
}

// eq. 30
// line by line with prefactors from eq. 8 incl.
double STUchecker::calc_S(){
  using exp_c::pi;
  double S0=0;
  for(size_t a = 1; a < n; ++a){
    S0 += pow(2*sw2-dUU(a),2)*G(mcsq[a],mcsq[a],mz2);
  }
  for(size_t a1 = 1; a1 < n-1; ++a1){
    for(size_t a2 = a1 + 1; a2 < n; ++a2){
      S0 += 2*UUsq(a1,a2)*G(mcsq[a1],mcsq[a2],mz2);
    }
  }
  for(size_t b1 = 1; b1 < m-1; ++b1){
    for(size_t b2 = b1 + 1; b2 < m; ++b2){
      S0 += ImVVsq(b1,b2)*G(m0sq[b1],m0sq[b2],mz2);
    }
  }
  for(size_t a = 1; a < n; ++a){
    S0 -= 2*dUU(a)*log(mcsq[a]);
  }
  for(size_t b = 1; b < m; ++b){
    S0 += dVV(b)*log(m0sq[b]);
  }
  S0 -= log(mhref2);
  for(size_t b = 1; b < m; ++b){
    S0 += ImVVsq(0,b)*G2(m0sq[b],mz2);
  }
  S0 -= G2(mhref2,mz2);
  return S0/24/pi;
}

// eq. 31
// line by line with prefactors from eq. 8 incl.
double STUchecker::calc_U(){
  using exp_c::pi;
  double U0=0;
  for(size_t a = 1; a < n; ++a){
    for(size_t b = 1; b < m; ++b){
      U0 += UVsq(a,b)*G(mcsq[a],m0sq[b],mw2);
    }
  }
  for(size_t a = 1; a < n; ++a){
    U0 -= pow(2*sw2-dUU(a),2)*G(mcsq[a],mcsq[a],mz2);
  }
  for(size_t a1 = 1; a1 < n-1; ++a1){
    for(size_t a2 = a1 + 1; a2 < n; ++a2){
      U0 -= 2*UUsq(a1,a2)*G(mcsq[a1],mcsq[a2],mz2);
    }
  }
  for(size_t b1 = 1; b1 < m-1; ++b1){
    for(size_t b2 = b1 + 1; b2 < m; ++b2){
      U0 -= ImVVsq(b1,b2)*G(m0sq[b1],m0sq[b2],mz2);
    }
  }
  for(size_t b = 1; b < m; ++b){
    U0 += ImVVsq(0,b)*(G2(m0sq[b],Mw2)-G2(m0sq[b],mz2));
  }
  U0 -= G2(mhref2,mw2)-G2(mhref2,mz2);
  return U0/24/pi;
}
