#include "ScannerSModelclasses.h"
#include "ScannerSAux.h"
#ifdef VERBOSE
#include "ScannerSVerbose.h"
#endif

using namespace std;

//Routine to generate a VEV point uniformly
void PhiVec::generate(){
  for(vector<double>::iterator iter=Vec.begin(),iter1=Vecmin.begin(),iter2=Vecmax.begin();iter!=Vec.end();)
    *iter++=r(*iter1++,*iter2++);//generate uniformly
}

//Routine to generate a VEV point uniformly
void PhiVec::generate(double (*pdf)(const std::vector<double> &)){
  const unsigned nmax=100000;
  unsigned i=0;
  for(;i!=nmax;++i){
    for(vector<double>::iterator iter=Vec.begin(),iter1=Vecmin.begin(),iter2=Vecmax.begin();iter!=Vec.end();)
      *iter++=r(*iter1++,*iter2++);//generate uniformly
    if(r()<pdf(Vec))
      break;
  }
  if(i>=nmax){
    cerr << "ERROR in void PhiVec::generate(double (*pdf)(const std::vector<double> &)): Too many tries. nmax = "<<nmax << endl;
    exit(EXIT_FAILURE);
  }
}

void PhiVec::setphi(const std::vector<double> & phi_in){
  if(phi_in.size()==Vec.size())
    Vec=phi_in;
  else
    std::cerr<< "ERROR in PhiVec::setphi(const std::vector<double> & phi_in) - vector sizes don't match. Set failed."<< endl;
}

std::istream & operator>>(std::istream & in, Potential & V){
  V.readfile(in); //the operator calls a function specific to the potential model
  return in;
} //Overloaded input operator to initialise

void Potential::readfile(std:: istream & in){
//virtual function, in worked models it opens specific potential files with openinput and then replaces relevant parameters
  openinput(in);
}

void Potential::openinput(std::istream & in){
  if(Mon.size()!=0)
    std::clog<< "Warning! Your matrix of monomials was not empty. Clearing matrix before proceeding..."<< endl;
  Mon.clear();
  in>>Phi>>PhiPar>>MixingPars>>MLpars>>Cp;	//Cp also contain the masses
  for(Potential::size_type i=0;i!=Cp.size();++i){
    Potential::size_type Nmon;
    in>>Nmon;
    std::vector<Monomial> VecMon;
    Monomial M_in(Phi.size());
    for(Potential::size_type j=0;j!=Nmon; ++j){
      in>>M_in;
      VecMon.push_back(M_in);
    }
    Mon.push_back(VecMon);
  }

  //temporary vectors to build matrices
  vector<double> tempV(Cp.size(),0);
  vector< vector<double> > tempdV(Phi.size(),tempV);
  vector< vector< vector<double> > > tempd2V(Phi.size(),tempdV);
  dV=tempdV;
  d2V=tempd2V;
  d2Vnew=tempd2V;
  d2V.Mag=tempV;
  d2Vnew.Mag=tempV;
  vector<unsigned int> tempOrder(Cp.size(),0);
  NIC=0;
  orderdV=tempOrder;
  orderd2V.resize(Cp.size()+Phi.size());
  TempM =  new double [Phi.size()*Phi.size()];
  TempM2 =  new double [Phi.size()*Phi.size()];
  TempOut = new double [(Phi.size()+1)*Phi.size()];
  vector<double> tempVphi(Phi.size(),0);
  vector< vector<double> > tempMatphi(Phi.size(),tempVphi);
  vector<unsigned int> tempBlocks(Phi.size(),0);
  vector<int> tempBlocks2(Phi.size(),0);
  BlockInfo=tempBlocks2;
  BlockPermutLabels=tempBlocks;
  Mbasis=tempMatphi;
  Maux=tempMatphi;
  Mrot=tempMatphi;
  Mrot2=tempMatphi;
  Mmixing=tempMatphi;
  a0tree.resize((Phi.size()*(Phi.size()+1))/2);
  for(size_t i=0;i!=(Phi.size()*(Phi.size()+1))/2;++i){
    a0tree[i].resize((Phi.size()*(Phi.size()+1))/2);
    for(size_t j=0; j!=(Phi.size()*(Phi.size()+1))/2;++j)
      a0tree[i][j].resize(Cp.size());
  }
  a0treeIni();
  a0treeSum = new double [(Phi.size()*(Phi.size()+1))/2*(Phi.size()*(Phi.size()+1))/2];

}

std::ostream & operator<<(std::ostream & os, const Potential & V){

  os<< " V(";
  for(Potential::size_type i=0;i!=V.Phi.size();++i){
    if(i!=0)
      os<<",";
    os<<"phi"<<i;
  }
  os<<") = ";

  for(Potential::size_type a=0; a!=V.Mon.size();++a){
    if(a!=0)
      os<<" + ";
    os<< "L"<<a<<"(";
    for(Potential::size_type b=0; b!=V.Mon[a].size();++b){
      if(b!=0)
	if(V.Mon[a][b].coeff_get()>0)
	  os<<" + ";
	else
	  os<<" ";
      os<<V.Mon[a][b];
    }
    os<<")";
  }

  return os;
}

std::ostream & operator<<(std::ostream & os, const PhiRef & P){
  os << P.Phi_;
  return os;
}

std::ostream & operator<<(std::ostream & os, const LambdaRef & L){
  os << L.L_;
  return os;
}

double Potential::operator()(std::vector< std::vector<Monomial> >::const_iterator aiter) const{
  //Routine to compute the Potential term directly from the polynomial expression

  double subsum=0e0;
  for(std::vector<Monomial>::const_iterator miter = (*aiter).begin(); miter!=(*aiter).end();++miter){
    double prod=(*miter).coeff_get();
    for(size_type j=0; j!=Phi.size(); ++j)
      {
	// Multiply by the appropriate power of the field if the power is positive (otherwise we assume the field is not present in the monomial)
	if((*miter)[j]>0)
	  prod*=pow(Phi[j],(*miter)[j]);
      }
    // Add monomial to the sum
    subsum+=prod;
  }

  return subsum;
}

double Potential::operator()(size_type a) const{
  std::vector< std::vector<Monomial> >::const_iterator aiter = Mon.begin()+a;
  if((a+1)>0 && a<Mon.size())
    return (*this)(aiter);
  else{
    cerr<< "ERROR in function: double Potential::operator[](size_type a) const -- Out of bounds, a= "<< a << " dimension = "<<Mon.size() << endl;
    exit(EXIT_FAILURE);
  }
}

double Potential::operator()() const{ 
  //Routine to compute the Potential directly from the polynomial expression
  // Start sum of polynomial terms
  double sum=0e0;
  for(std::vector< std::vector<Monomial> >::const_iterator aiter = Mon.begin();aiter!=Mon.end();++aiter){
    double subsum=(*this)(aiter-Mon.begin());
    sum+=subsum * Cp[aiter-Mon.begin()];
  }
  return sum;
}

double Potential::operator()(size_type i, size_type a) const{
    //Routine to compute the derivative of the Potential directly from the polynomial expression for the a_th component.

  //Start sum of monomials
  double sum=0e0;
  //Loop over  all terms for the a_th component
  for(size_type k=0; k!= Mon[a].size();++k){
    //Start loop to multiply all powers in monomial
    // First multiply by numerical factor...
    double prod=Mon[a][k].coeff_get();
    // Then if the power for the field with respect to which we are differentiating is positive multiply by the corresponding term
    if(Mon[a][k][i]>0){
      for(size_type j=0; j!=Mon[a][k].size(); ++j)
	{
	  //If we are looking at the field which we are differentiating w.r.t...
	  if(j==i){
	    prod*=Mon[a][k][j]*pow(Phi[j],Mon[a][k][j]-1);
	  }//Otherwise multiply by corresponding term...
	  else if(Mon[a][k][j]>0)
	    prod*=pow(Phi[j],Mon[a][k][j]);
	}
    }//Otherwise mutiply by zero because the monomial doesn't depend on the field
    else
      prod=0e0;
    //Sum monomials...
    sum+=prod;
  }
  //Return final value for derivative...
  return sum;
}

double Potential::operator()(size_type i, size_type j,size_type a) const{
  //Routine to compute the second derivative of the Potential directly from the polynomial expression for the a_th component.

  //Start sum of terms...
  double sum=0e0;
  for(size_type k=0; k!=Mon[a].size();++k){
    //Start products
    double prod=Mon[a][k].coeff_get();
    //Case of second order derivative w.r.t. same phi
    if(i==j){
      // Do as before and compute if power is large enough for phi being deffirentiated w.r.t.
      if(Mon[a][k][j]>1){
	for(size_t j1=0; j1!=Mon[a][k].size(); ++j1)
	  {
	    // Field beig differentiated...
	    if(j1==j){
	      prod*=Mon[a][k][j1]*(Mon[a][k][j1]-1)*pow(Phi[j1],Mon[a][k][j1]-2);
	    }//Other fields
	    else if(Mon[a][k][j1]>0)
		prod*=pow(Phi[j1],Mon[a][k][j1]);
	  }
      }// Set to zero if power not large enough
      else
	prod=0e0;
    }//Mixed derivatives case
    else {
      //Check that powers are large enough for both fields in the mixed derivative...
      if(Mon[a][k][j]>0 && Mon[a][k][i]>0){
	for(size_type j1=0; j1!=Mon[a][k].size(); ++j1)
	  {//Fields being differentiated case...
	    if(j1==j || j1== i){
	      prod*=Mon[a][k][j1]*pow(Phi[j1],Mon[a][k][j1]-1);
	    }//Other fields...
	    else if(Mon[a][k][j1]>0)
		prod*=pow(Phi[j1],Mon[a][k][j1]);
	  }
      }// Not enough powers...
      else
	prod=0e0;
    }
    //Sum all terms...
    sum+=prod;
  }
  return sum;
}

void rules_fast(){

  //1) Create rules to solve vacuum linear conditions

  //2) Define rules to reduce second derivative matrices by first set of couplings

  //3) Find block diagonal structure of mixing, i.e. define reference basis

  //4) Find rules to solve quadratic conditions and identify independent couplings

  //Now the procedure can be iterated backwards without repeating the solution of the equations

}

void check_rules_fast_stable(){

}

bool Potential::generate(){
  int hcont=0;
  do{
    if(PhiPar.size()>0){
      PhiPar.generate();
      MyPhiParametrization(PhiPar,Phi);
    }
    else{//generate Phi
      Phi.generate();
    }
    //Compute first derivative matrix
    for(vector< vector<double> >::iterator iter=dV.begin();iter!=dV.end();++iter)
      for(vector<double>::iterator ater=iter->begin();ater!=iter->end();++ater)
		*ater=(*this)(iter-dV.begin(),ater-iter->begin());

#ifdef DEBUG
    /////Check: Print out matrix (in verbose mode)
    std::clog << endl << "**************** Print out dV ******************";
    printdV();
#endif

    //Perform Step 1 of gaussian elimination
    hcont=GaussUpTriang(dV);
#ifdef DEBUG
    /////Check: Print out matrix (in verbose mode)
    std::clog << endl << "****** Print out dV after step 1 of gaussian elimination ********";
    printdV();
#endif
  }while(hcont==0 && dV.size()>=dV[0].size() && (std::clog<<"WARNING: Massless theory with kinetic terms only generated. Trying again..."<< endl));

  //Perform step 2 of Gaussian elimination
  GaussSolveUpTriang(dV,NIC,orderdV,hcont);
#ifdef DEBUG
  /////Check: Print out matrix (in verbose mode)
  std::clog << endl << "****** Print out dV after step 2 of gaussian elimination ********";
  printdV();
  std::clog << "Number of independent linear conditions = " << NIC << endl;
  std::clog << endl<< "******** Ordering of couplings ***********"<< endl;
  for(size_t a=0;a!=dV[0].size();a++){
    std::clog <<" dV[" << a<<"] = " <<orderdV[a] << endl;
  }
#endif

  //Compute second derivative matrices
  for(vector< vector< vector<double> > >::iterator iter=d2V.begin();iter!=d2V.end();++iter)
    for(vector< vector<double> >::iterator jter=iter->begin();jter!=iter->end();++jter)
      for(vector<double>::iterator ater=jter->begin();ater!=jter->end();++ater)
	*ater=(*this)(iter-d2V.begin(),jter-iter->begin(),ater-jter->begin());

#ifdef DEBUG
    /////Check: Print out matrix (in verbose mode)
    std::clog << endl << "**************** Print out d2V[a][i][j] ******************";
    printd2V();
    std::clog << "Analysing quadratic vacuum conditions ..." << endl;
#endif

    RedBydV(dV,d2V,NIC,orderdV);
#ifdef DEBUG
    ///////////////!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!////////////
    std::clog << endl << "****** d2V (Hessians) reduced by linear condition *******" << endl;
    PrintM21(d2V,NIC,orderdV);
    std::clog << endl<< "****** d2V print out for comparison with initial matrices ******" << endl;
    PrintM22(d2V,NIC,orderdV);

    std::clog << endl<< endl << "****** Checking for a priori eigen directions ..." << endl;
#endif
    if(FindMixing()){
#ifdef VERBOSE
      printpoint();
#endif

      //Compute mixing matrix
      ////Part 1 which involves Mrot block
      for(size_t i=0;i!=Nmixed; ++i){
	for(size_t j=0;j!=d2V.size(); ++j){
	  Mmixing[i][j]=0e0;
	  for(size_t k=0;k!=Nmixed;++k){
	    Mmixing[i][j]+=Mrot[i][k]*Mbasis[k][j];
	  }
	}
      }
      ////Part 2 where Mrot is the identity
      for(size_t i=Nmixed;i!=d2V.size(); ++i){
	for(size_t j=0;j!=d2V.size(); ++j){
	  Mmixing[i][j]=Mbasis[i][j];
	}
      }

      unsigned int NL=d2V[0][0].size()-NIC+Nmixed+NnoMixCurv;
      Cp.Lindp.resize(0);
      Cp.Mindp.resize(0);
      for(size_t A=NIC2; A!=NL;++A){
	if(orderd2V[A]<d2V[0][0].size()-NIC){
	  size_t ilambda=orderdV[NIC+orderd2V[A]];
	  Cp.L[ilambda]=r(Cp.L.vgetmin(ilambda),Cp.L.vgetmax(ilambda));
	  // Ltemp[orderd2V[A]]=Cp.L[ilambda]; (done below! was a bug)
	  Cp.Lindp.push_back(ilambda);
	}
	else{
	  size_t iphi=orderd2V[A]-(d2V[0][0].size()-NIC);
	  Cp.Mass[iphi]=r(Cp.Mass.vgetmin(iphi),Cp.Mass.vgetmax(iphi));
	  //Ltemp[orderd2V[A]]=pow(Cp.Mass[iphi],2); (done below! was a bug)
	  Cp.Mindp.push_back(iphi);
	}
      }
#ifdef VERBOSE
      for(size_t A=NIC2; A!=NL;++A){
	if(orderd2V[A]<d2V[0][0].size()-NIC){
	  size_t ilambda=orderdV[NIC+orderd2V[A]];
	  std::clog<<"Coupling "<<ilambda<< " is independent."<< endl;
	}
	else{
	  size_t iphi=orderd2V[A]-(d2V[0][0].size()-NIC);
	  std::clog<<"Mass of state "<<iphi<< " is independent." <<endl;
	}
      }
#endif

      //Check if reparametrization of independent Masses/Couplings is active
      if(MLpars.size()>0){
	/////std::clog<<"Using parametrisations"<<endl;
	/////std::clog<<"Current masses "<<Cp.Mass[3]<<"\t "<<Cp.Mass[4]<<" Current L7 = "<<Cp.L[7]<<endl;
	vector<double> IndependentMs, IndependentLs,Lvec(Cp.L.size()),Mvec(Cp.Mass.size());//Auxiliary variables
	for(size_t i=0;i!=Cp.Mass.size();++i)
	  Mvec[i]=Cp.Mass[i];
	for(size_t i=0;i!=Cp.L.size();++i)
	  Lvec[i]=Cp.L[i];
	//Compute re-parametrized fields
	for(size_t i=0; i!=MLpars.size();++i){
	  MLpars[i]=r(MLpars.vgetmin(i),MLpars.vgetmax(i));//Generate parameters uniformly in range decided in mathematica file
	}
	//Compute new Masses and Couplings with the user defined function

	MyCoupMassRelations(PhiPar,Phi,Mmixing,MixingPars,Lvec,Mvec,MLpars,IndependentMs,IndependentLs);
	bool testML=true; //Variable to test if the choice of masses and couplings to re-parametrize is ok. Start by assuming it is and flip tp false whenever a choice is not possible
	//Check if choice is ok for masses
	for(size_t i=0;i!=IndependentMs.size();++i){
	  bool testM=false;//Assume the test will fail
	  for(size_t j=0;j!=Cp.Mindp.size();++j){
	    if(IndependentMs[i]==Cp.Mindp[j])
	      testM=true;//If the number matches one of the independent masses flip to true
	  }
	  testML*=testM; //if no match is found testM will be false so testML will also become false
	}
	//Check if choice is ok for Couplings (same procedure!)
	for(size_t i=0;i!=IndependentLs.size();++i){
	  bool testL=false;
	  for(size_t j=0;j!=Cp.Lindp.size();++j){
	    if(IndependentLs[i]==Cp.Lindp[j])
	      testL=true;
	  }
	  testML*=testL;
	}
	if(testML){//If test succeeds, then asign new re-parametrized mass & coupling values to the corresponding independent coupplings
	  for(size_t i=0; i!=IndependentMs.size();++i){
	    Cp.Mass[IndependentMs[i]]=Mvec[IndependentMs[i]];
	    /////////std::clog<<"Mass "<< IndependentMs[i]<<" changed to "<< Cp.Mass[IndependentMs[i]] <<" using "<< Mvec[IndependentMs[i]]<< endl;
	  }
	  for(size_t i=0; i!=IndependentLs.size();++i)
	    Cp.L[IndependentLs[i]]=Lvec[IndependentLs[i]];
	}
	else{
	  std::clog<<"WARNING: You have chosen invalid variables to re-parametrise at the last stage of the generation of the point (IndependentMs in MyCoupMassRelations). The scan will continue ignoring this re-parametrisation. Please correct your ScannerSUser.cpp if you wish to use the re-parametrisation."<< endl;
	}
      }

      // std::clog<<"After params masses "<<Cp.Mass[3]<<"\t "<<Cp.Mass[4]<<" Current L7 = "<<Cp.L[7]<<endl;

      //Set Goldstone masses to zero
      for(size_t ip=Nmixed+NnoMixCurv;ip!=d2V.size();++ip)
	Cp.Mass[ip]=0;

      vector<double> Ltemp(NL); //Temporary vector of couplings and masses to compute dependent couplings
      for(size_t A=NIC2; A!=NL;++A){
	if(orderd2V[A]<d2V[0][0].size()-NIC)
	  Ltemp[orderd2V[A]]=Cp.L[orderdV[NIC+orderd2V[A]]];
	else
	  Ltemp[orderd2V[A]]=pow(Cp.Mass[orderd2V[A]-(d2V[0][0].size()-NIC)],2);
      }

      // Determine dependent couplings
      for(size_t A=0;A!=NIC2; ++A){
	if(orderd2V[A]<d2V[0][0].size()-NIC){
	  size_t ilambda=orderdV[NIC+orderd2V[A]];
	  double sum=0e0;
	  for(size_t j=NIC2;j!=NL;++j){
	    //std::clog<<"index A: "<<A<<" d2Vreduced.size: "<<d2Vreduced.size()<<" index orderd2V[j]: "<<orderd2V[j];
	    //std::clog<<" d2Vreduced[A].size: "<<d2Vreduced[A].size()<<" index j: "<<j<<" orderd2V.size: "<<orderd2V.size()<<endl;
	    sum-=d2Vreduced[A][orderd2V[j]]*Ltemp[orderd2V[j]];
	  }

	  Cp.L[ilambda]=sum;
	}
	else{
	  size_t iphi=orderd2V[A]-(d2V[0][0].size()-NIC);
	  double sum=0e0;
	  for(size_t j=NIC2;j!=NL;++j){
	    sum-=d2Vreduced[A][orderd2V[j]]*Ltemp[orderd2V[j]];
	  }
	  if(sum<0){
#ifdef VERBOSE
	    std::clog<<endl<<"---> ScannerS: Point rejected by Potential (sum<0)::generate(). sum is "<<sum<<endl;
#endif
	    return false;
	  }
	  //std::clog<<"iphi=..."<<iphi<<" mass= "<<sqrt(abs(sum))<<endl;
	  Cp.Mass[iphi]=sqrt(abs(sum));
	}
      }

      for(size_t a=0;a!=NIC;++a){
	size_t ilambda = orderdV[a];
	double sum=0e0;
	for(size_t j=NIC;j!=d2V[0][0].size();++j){
	  sum-=dV[a][orderdV[j]]*Cp.L[orderdV[j]];
	}
	Cp.L[ilambda]=sum;
      }


      for(size_t i=0;i!=Cp.L.size();++i)
	if (!is_valid(Cp.L[i])){
	  cerr<<"ScannerS Warning - In Potential::generate(): Coupling L"<<i<<" was generated with a non-valid number. The point will be rejected."<<endl;
	  return false;
	}
      for(size_t i=0;i!=Cp.Mass.size();++i)
	if (!is_valid(Cp.Mass[i])){
	  cerr<<"ScannerS Warning - In Potential::generate(): Mass "<<i<<" was generated with a non-valid number. The point will be rejected."<<endl;
	  return false;
	}
      for(size_t i=0;i!=Phi.size();++i)
	if (!is_valid(Phi[i])){
	  cerr<<"ScannerS Warning - In Potential::generate(): Phi"<<i<<" was generated with a non-valid number. The point will be rejected."<<endl;
	  return false;
	}
      for(size_t i=0;i!=Mmixing.size();++i)
	for(size_t j=0;j!=Mmixing[i].size();++j)
	  if (!is_valid(Mmixing[i][j])){
	    cerr<<"ScannerS Warning - In Potential::generate(): Mix["<<i<<"]["<<j<<"] was generated with a non-valid number. The point will be rejected."<<endl;
	    return false;
	  }


      ReCalcMixing();// Function to re-compute the basis in case there are residual arbitrary rotations due to the numerical procedure.


#ifdef VERBOSE
      std::clog << endl << "----------------------";
      std::clog << endl << "--- Mixing matrix ----";
      std::clog << endl << "----------------------";
      std::clog << endl << "Note: M_{jk} -> indices j (lines) are mass eigenstates and k (columns) are original states" << endl;

      PrintM1(Mmixing);
      std::clog << endl << "----------------------";
      std::clog << endl << "------ Masses --------";
      std::clog << endl << "----------------------";
      std::clog << endl << "Note: Mass eigenstates ordered as in the description above (see NEW BASIS section above for block basis of states that were mixed, or states that were found to be eigenstates a priori)" << endl;
      for(int i=0; i<Cp.Mass.size();++i)
	std::clog<< "M["<<i<<"]="<<Cp.Mass[i]<< endl;

      std::clog << endl << "----------------------";
      std::clog << endl << "------ Couplings -----";
      std::clog << endl << "----------------------" << endl;

      for(int i=0; i<Cp.L.size();++i)
	std::clog<< "L["<<i<<"]="<<Cp.L[i]<< endl;

      std::clog << endl << "----------------------";
      std::clog << endl << "-------- VEVs --------";
      std::clog << endl << "----------------------" << endl;

      for(int i=0; i<Phi.size();++i)
	std::clog<< "Phi["<<i<<"]="<<Phi[i]<< endl;


#endif
      
      //return true;
      return UserCheckLocalMin();//User defined check if the local minimum is as (s)he wants.
    }
    else{
#ifdef VERBOSE
      std::clog<<endl<<"---> ScannerS: Point rejected by Potential::generate()."<<endl;
#endif
      return false;
    }

}

void Potential::setprint(int num_points,int data_point){
//Sets up print options (how many points must be store before printing and how much data is per point
  store_vector_index=0;
  points_per_print=num_points;
  data_per_point=data_point;
  points2print.resize(num_points);
  vector<double> temp(data_point);
  for(size_t i=0;i!=data_point;++i){
    temp[i]=0.;
  }
  for(size_t i=0;i!=num_points;++i){
    points2print[i]=temp;
  }
}

void Potential::print_or_store(std::ostream & out){
//stores points up to num_point in point2print, then prints them to a file
  what2print();
  if(store_vector_index==points_per_print-1){
    for(size_t i=0;i!=points_per_print;++i){
      for(size_t j=0;j!=data_per_point;++j){
	out<<points2print[i][j]<<"\t";
	points2print[i][j]=0.;
      }
      out<<endl;
    }
    store_vector_index=0;
  }
  else
    ++store_vector_index;
}

void Potential::print_remaining_points(std::ostream & out){
//prints the remaining points after the scan (in case the full scan was not a multiple of points_per_print)

    for(size_t i=0;i!=store_vector_index;++i){
      for(size_t j=0;j!=data_per_point;++j){
	out<<points2print[i][j]<<"\t";
	points2print[i][j]=0.;
      }
      out<<endl;
    }
}

void Potential::printpoint(){
// Using only in verbose mode
  std::clog << endl ;
  std::clog << endl << "********************************************************" ;
  std::clog << endl << "********************************************************" ;
  std::clog << endl << "******** SUMMARY INFO FOR THIS ATTEMPTED POINT *********" ;
  std::clog << endl << "********************************************************" ;
  std::clog << endl << "********************************************************" << endl;
  std::clog << endl << "------------------";
  std::clog << endl << "--- New basis ----";
  std::clog << endl << "------------------" << endl;

  std::clog << endl << "--- Blocks which will mix ---" << endl;
  int sumsizes=0;
  for(size_t a=0;a!=BlockDims.size();++a){
	std::clog<< "*** Block "<<a<< endl;
	for(size_t i=sumsizes; i!=BlockDims[a]+sumsizes;++i){
	  std::clog << "v["<<i<<"]= ";
	  for(size_t j=0;j!=d2V.size();++j){
	    std::clog<<Mbasis[i][j]<< " dphi"<<j<<" + ";
	  }
	  std::clog << endl;
	}
	sumsizes+=BlockDims[a];
  }
  int ndegen=0;
  for(size_t i=0;i!=degeneracies.size();++i)
	ndegen+=degeneracies[i];
  std::clog << endl << "--- Non-degenerate Curved diagonal directions ---" << endl;
  for(size_t i=Nmixed; i!=Nmixed+NnoMixCurv-ndegen; ++i){
	std::clog << "v["<<i<<"]= ";
	for(size_t j=0;j!=d2V.size();++j)
	  std::clog<<Mbasis[i][j]<< " dphi"<<j<<" + ";
	std::clog << endl;
  }
  std::clog << endl << "--- degenerate Curved diagonal directions ---" << endl;
  int sumdeg=0;
  for(size_t b=0; b!=degeneracies.size();++b){
	std::clog<< "*** Group "<< b<< endl;
	for(size_t i=Nmixed+NnoMixCurv-ndegen+sumdeg; i!=Nmixed+NnoMixCurv-ndegen+sumdeg+degeneracies[b]; ++i){
	  std::clog << "v["<<i<<"]= ";
	  for(size_t j=0;j!=d2V.size();++j)
	    std::clog<<Mbasis[i][j]<< " dphi"<<j<<" + ";
	  std::clog << endl;
	}
	sumdeg+=degeneracies[b];
  }

  std::clog << endl << "--- Goldstone directions ---" << endl;
  for(size_t i=Nmixed+NnoMixCurv; i!=d2V.size(); ++i){
	std::clog << "v["<<i<<"]= ";
	for(size_t j=0;j!=d2V.size();++j){
	  std::clog<<Mbasis[i][j]<< " dphi"<<j<<" + ";
	}
	std::clog << endl;
  }
  std::clog<<endl;
  std::clog<<"---------------------------------------------------------------"<< endl;
  std::clog<<"--- Independent parameters generated after VEVs and Mixings ---"<< endl;
  std::clog<<"---------------------------------------------------------------"<< endl<< endl;
}

bool Potential::FindMixing(){
  unsigned int kflat=0; //Counts number of flat directions
  unsigned int kcurved=0; //Counts number of flat directions

  FindAprioriEigenDirs(d2V,Mbasis,TempM,TempM2,TempOut,orderdV,degeneracies,BlockInfo,BlockDims,kflat,kcurved,NIC);

#ifdef DEBUG
    //Print out////////!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!////////////
  std::clog << endl << "Found " << kflat<< " a priori flat directions" ;
  for(size_t k=d2V.size()-kflat;k!=d2V.size(); ++k){
    //Print out flat direction
    std::clog << endl;
    for(int i=0;i!=d2V.size();i++){
      std::clog << Mbasis[k][i]<< "\t";
    }
    if(k<d2V.size()-1){
      double dot=0;
      std::clog<< endl;
      for(int i=0;i!=d2V.size();i++){
	dot+=Mbasis[k][i]*Mbasis[k+1][i];
      }
      std::clog<<"dot product with next vector is "<< dot<< endl;
    }
  }
  std::clog << endl;
  std::clog << endl << "Found " << kcurved << " a priori curved eigen-directions" << endl ;

  for(int k=d2V.size()-kflat-kcurved;k!=d2V.size()-kflat; ++k){
    std::clog << endl;
    for(int i=0;i!=d2V.size();++i)
      std::clog << Mbasis[k][i]<< "\t";
  }
  std::clog << endl;
  std::clog<< "degeneracy info vector:"<< endl;
  for(size_t i=0; i<degeneracies.size();++i)
    std::clog<<degeneracies[degeneracies.size()-i-1]<< "\t";

  std::clog << endl << "Found " << d2V.size()-kflat-kcurved << " mixing directions" << endl ;

  for(int k=0;k!=d2V.size()-kflat-kcurved; ++k){
    std::clog << endl;
    for(int i=0;i!=d2V.size();++i)
      std::clog << Mbasis[k][i]<< "\t";

    if(k<d2V.size()-kflat-kcurved-1){
      double dot=0;
      std::clog<< endl;
      for(int i=0;i!=d2V.size();i++){
	dot+=Mbasis[k][i]*Mbasis[k+1][i];
      }
    }

  }
  std::clog << endl;

#endif
  ConvertEigen(d2V,d2Vnew,Mbasis,orderdV,kflat,kcurved,NIC);


#ifdef DEBUG
    ////////////////////////CHECK!!!!!!!!!!!!!!!!!!!!
  std::clog << endl<< "****** d2Vnew independent matrices in new basis  ******" << endl;
  PrintM22(d2Vnew,NIC,orderdV);
#endif
  //exit(EXIT_FAILURE);
  unsigned int kmix=d2V.size()-kflat-kcurved;


  if(MixingPars.size()>0){
    //reference vector with positions of mixing gauge states
    vector<int> statesmixing;
    //Detect mixing gauge states for first mixing direction
    for(size_t j=0;j<Mbasis.size();++j){
      if(Mbasis[0][j]!=0)
	statesmixing.push_back(j);
    }
    //Detect mixing gauge states for other mixing direction (without repetition)
    for(size_t i=1;i<kmix ;++i){
      for(size_t j=0;j<Mbasis.size();++j)
	if(Mbasis[i][j]!=0 && j> statesmixing.back()){//avoid counting again states that are already on the list and insert the ones that were not present in the previous vector
	  statesmixing.push_back(j);
	}
    }
    if(statesmixing.size()==1 && kmix==0){
      //std::clog<<endl<<"ScannerS WARNING (doing nothing.....): In Potential::FindMixing(), for this model the code has detected no mixing, which is consistent with your 1 by 1 mixing matrix parametrisation. Please remove your parametrisation since it is not necessary (the code will identify all eigenstates automatically)."<<endl;
      //exit(EXIT_FAILURE);
    }
    else if(statesmixing.size()!=kmix){
      std::clog<<endl<<"ScannerS Warning: ERROR in bool Potential::FindMixing(): Number of scalar states mixing ("<<statesmixing.size()<<") is different from the number of detected mixing directions ("<<kmix<<"), so user defined mixing matrix is ambiguous!" << endl;
      std::clog<<"This sometimes happens due to numerical errors that create infinitesimal values in parameters that should be 0."<<endl;
      std::clog<<"This point will be ignored."<<endl;
	  return false;
    } 
    else{
      //Generate parameters
      for(size_t i=0; i!=MixingPars.size();++i){
	MixingPars[i]=r(MixingPars.vgetmin(i),MixingPars.vgetmax(i));//Generate parameters uniformly in range decided in mathematica file
      }
      //Calculate mixing matrix according to user or model defined expressions (its a virtual function)
      MyInternalMixing(PhiPar,Phi,MixingPars,Mrot);
      /////TO DO!!!!! CHECK THAT THIS IS REALLY NOT NECESSARY WITH THE NEW BASIS DETECTION ADD ON!!!!!!!
      /*// Rotate Mrot backwards on the right, to cancel out the arbitrary basis that was used in the eigen-directions search
	for(size_t i=0;i<kmix;++i)
	for(size_t j=0;j<kmix;++j){
	Mrot2[i][j]=0;
	for(size_t p=0;p<kmix;++p)
	Mrot2[i][j]+=Mrot[i][p]*Mbasis[j][statesmixing[p]];//Fill up temporary matrix with "right"-rotated matrix
	}
	for(size_t i=0;i<kmix;++i)
	for(size_t j=0;j<kmix;++j){
	Mrot[i][j]=Mrot2[i][j];//Set elements of the new rotation matrix
	}*/
      
      ConvertWithMrot(d2Vnew,Mrot,orderdV,kmix,NIC);
    }
  }
  else{
    //Initialize Mrot to zero
    for(size_t i=0; i!=kmix;++i)
      for(size_t j=0; j!=kmix;++j)
	Mrot[i][j]=0;

    int sumdims=0;
    for(size_t b=0;b!=BlockDims.size();++b){
      GenMrot(TempM,BlockDims[b]);
      //Save matrix in Mrot
      for(size_t i=0; i!=BlockDims[b];++i)
	for(size_t j=0; j!=BlockDims[b];++j)
	  Mrot[sumdims+i][sumdims+j]=TempM[BlockDims[b]*i+j];
      sumdims+=BlockDims[b];
    }

    ConvertWithMrot(d2Vnew,Mrot,orderdV,kmix,NIC);
  }

#ifdef DEBUG
    ////////////////////////CHECK!!!!!!!!!!!!!!!!!!!!
  std::clog << endl<< "****** Internal mixing matrix ****** " << endl;
  for(size_t i=0;i<kmix;++i){
    for(size_t j=0;j<kmix;++j)
      std::clog<<Mrot[i][j]<< "\t";
    std::clog<< endl;
  }
  std::clog<< endl;
#endif

  //unsigned int kcurved=d2Vnew.size()-(kflat-kminus)-kminus2;
  unsigned int Nij=(kmix*(kmix+1))/2+kcurved; //Number of equations in the system (lines)
  unsigned int NL=d2Vnew[0][0].size()-NIC+kmix+kcurved; //Number of variables (columns)



  //Re-sizing matriz that will be used for the calculations to the correct dimensions
  d2Vreduced.resize(Nij);
  for(size_t i=0;i!=Nij;++i)
    d2Vreduced[i].resize(NL);

  ///Set first conditions corresponding to equaling to the mass squared (diagonal conditions)
  for(size_t i=0;i!=kmix+kcurved;++i){
    //Set columns corresponding to couplings to their values
    for(size_t A=0; A!=d2Vnew[0][0].size()-NIC; ++A)
      d2Vreduced[i][A]=d2Vnew[i][i][orderdV[A+NIC]];
    //Set remaining columns to a "diagonal" form where the masses squared appear qith coefficient -1
    for(size_t A=d2Vnew[0][0].size()-NIC; A!=NL; ++A){
      if(A-(d2Vnew[0][0].size()-NIC)==i)
	d2Vreduced[i][A]=-1e0;
      else
	d2Vreduced[i][A]=0e0;
    }
  }

  ///Set other (off-diagonal) conditions from the mixed block
  unsigned int k=kmix+kcurved;
  for(size_t i=0;i!=kmix;++i){
    for(size_t j=i+1;j!=kmix; ++j){
      for(size_t A=0; A!=d2Vnew[0][0].size()-NIC; ++A)
	d2Vreduced[k][A]=d2Vnew[i][j][orderdV[A+NIC]];

      for(size_t A=d2Vnew[0][0].size()-NIC; A!=NL; ++A)
	d2Vreduced[k][A]=0e0;

      k++;
    }
  }

#ifdef DEBUG
  std::clog << endl<< "****** d2Vreduced final homogeneous system  ******" << endl;
  PrintM1(d2Vreduced);
#endif

  //Perform Gaussian elimination
  ////Initialise counter of horizontal continuations in gaussian elimination
  int hcont=0;
  ////Call Gaussian elimination routine to act on d2Vreduced
  hcont=GaussUpTriang(d2Vreduced);

#ifdef DEBUG
  /////Check: Print out matrix
  std::clog << endl << "****** Print out d2Vreduced after step 1 of gaussian elimination ********";
  PrintM1(d2Vreduced);
#endif

 /////////////////////////////////////
  // Case when the solution of the system is the null vector (massless theory with kinetic terms only --> discard)
  ///!!!!!!!!!!!!!!!!!!!!!introduced extra condition recently!!!! check!!!!
  if(hcont==0 && (Nij>=NL)){
    return false;
  }
  else{
    // Find position of variables to be eliminated in the second step, perform second step of Gaussian elimination

    GaussSolveUpTriang(d2Vreduced,NIC2,orderd2V,hcont);


#ifdef DEBUG
    /////////////////////////////////////////////////////////
    /////Check: Print out matrix
    std::clog << endl << "****** Print out d2Vreduced after step 2 of gaussian elimination ********";
    PrintM1(d2Vreduced);
    std::clog << "Number of independent linear conditions = " << NIC2 << endl;

#endif
  }

#ifdef DEBUG
  ///////////////////////////////////////////////
  ///////Check: Print out ordering of couplings
  std::clog << endl<< "******** Ordering of couplings ***********"<< endl;
  for(size_t A=0;A!=NL;++A){
    std::clog << "A = " << A << " d2Vreduced[" << A<<"] = " <<orderd2V[A] << endl;
  }
#endif
  ////////////////////////////////////////////////////////////

  //Fill integers with info about mixing and Goldstones
  Nmixed=kmix;
  NnoMixCurv=kcurved;
  NGold=d2Vnew.size()-NnoMixCurv-Nmixed;
  return true;
}

bool Potential::ReCalcMixing(){
  // This function attempts to change to the best possible basis within each block by assuming that the diagonalisation of the mass matrix (after all parameters are fixed) would naturally do so

  const double epsilon=1e-8;//error control switch

  //1) build mass matrix again using all the Ls and VEVs
  double d2Vrebuilt[Phi.size()*Phi.size()]; // Auxiliary array to rebuild mass matrix
  double d2Vrebuiltcopy[Phi.size()*Phi.size()]; // DELETE THIS COPY AT THE END OF DEBUG!!!!
  ////Initialise array to zero CHECK I CAN DELETE THIS!!!!!
  ///////////for(size_t K=0;K!=Phi.size()*Phi.size();++K)
  /////////////d2Vrebuilt[K]=0;

  ////Fill up array with relevant entries
  for(size_t i=0;i!=Phi.size();++i){
    for(size_t j=i;j!=Phi.size();++j){
      d2Vrebuilt[i*Phi.size()+j]=0e0;
      for(size_t a1=NIC;a1!=Cp.L.size();++a1)
	d2Vrebuilt[i*Phi.size()+j]+=d2V[i][j][orderdV[a1]]*Cp.L[orderdV[a1]];
      d2Vrebuiltcopy[i*Phi.size()+j]=d2Vrebuilt[i*Phi.size()+j];
      d2Vrebuiltcopy[j*Phi.size()+i]=d2Vrebuilt[i*Phi.size()+j];
      if(i!=j)
	d2Vrebuilt[j*Phi.size()+i]=d2Vrebuilt[i*Phi.size()+j];
    }
  }
#ifdef DEBUG

  std::clog<<"---------------------------------------------------------"<<std::endl;
  std::clog<<"------------- Re-calculating mass eigenbasis ------------"<<std::endl;
  std::clog<<"---------------------------------------------------------"<<std::endl;


  std::clog<<std::endl;
  std::clog<<"Printing out original masses squared matrix in weak basis for this point..."<<std::endl;
  for(size_t i=0;i!=Phi.size();++i){
    for(size_t j=0;j!=Phi.size();++j)
      std::clog<<d2Vrebuilt[i*Phi.size()+j]<<"\t";
    std::clog<<std::endl;
  }
#endif

  //2) compute eigensystem
  double d2VrebuiltEigen[Phi.size()*(Phi.size()+1)]; // Auxiliary array to hold results of diagonalisation
  EigenSystemRsym(Phi.size(),d2Vrebuilt,d2VrebuiltEigen); // Diagonalise

  //compute modulus of trace to use as scale for chopping masses.
  double trace=0;
  for(size_t i=0;i!=Phi.size();++i)
    trace+=abs(d2VrebuiltEigen[i]);
  trace/=Phi.size();//average out squared masses
  //Now chop small squared masses
  for(size_t i=0;i!=Phi.size();++i)
    if(abs(d2VrebuiltEigen[i])<epsilon*trace)
      d2VrebuiltEigen[i]=0e0;

  //Chop eigenvectors
  for(size_t i=0;i!=Phi.size();++i)
    for(size_t j=0;j!=Phi.size();++j)
      if(abs(d2VrebuiltEigen[(i+1)*Phi.size()+j])<epsilon)
	d2VrebuiltEigen[(i+1)*Phi.size()+j]=0e0;

#ifdef DEBUG
  std::clog<<std::endl;
  std::clog<<"Printing out results after applying Eigensystem again... "<<std::endl;
  std::clog<<"The new eigen-values are: (left: squared masses; right: square root of modulus)"<<std::endl;
  for(size_t i=0;i!=Phi.size();++i){
    std::clog<<d2VrebuiltEigen[i]<<"\t"<< sqrt(abs(d2VrebuiltEigen[i]))<<std::endl;
  }

  std::clog<<"The eigenvectors are:"<<std::endl;
  for(size_t i=0;i!=Phi.size();++i){
    for(size_t j=0;j!=Phi.size();++j){
      std::clog<<d2VrebuiltEigen[(i+1)*Phi.size()+j]<<"\t";
    }
    std::clog<<std::endl;
  }

#endif

  //3) match new basis to previously ordered masses (within numerical errors) and print it out

  // vector of new massive states positions
  vector<pair<double,size_t> > new_states;
  for(size_t i=NGold;i!=Phi.size();++i)
    new_states.push_back(pair<double,size_t>(d2VrebuiltEigen[i],i));

  // vector of positions of massive states
  vector<size_t> pos_new_basis(Phi.size()-NGold,0);
  for(size_t i=0;i!=Phi.size()-NGold;++i)//Looping through already ordered masses
    for(size_t j=0;j!=new_states.size();++j){
      if(abs(Cp.Mass[i]*Cp.Mass[i]-new_states[j].first)<100*epsilon*trace){//Allowing here for some extra tolerance to rounding errors
	pos_new_basis[i]=new_states[j].second;//saving position of new eigenvector for the same state
	new_states.erase (new_states.begin()+j);//remove state from search vector os positions
	break;
      }
    }

  if(new_states.size()!=0){
#ifdef VERBOSE
    std::cerr<<std::endl<<"ScannerS Warning - In Potential::generate(): When attempting to simplify basis. Redundant check of masses failed (probably due to round off errors). "<<std::endl;
#ifdef DEBUG
    std::cerr<<"Original masses are: ";
    for(size_t i=0;i!=Phi.size();++i){
      std::cerr<<Cp.Mass[i]<<"\t";
    }
    std::cerr<<std::endl<<"New masses are: ";
    for(size_t i=0;i!=Phi.size();++i){
      std::cerr<<sqrt(abs(d2VrebuiltEigen[i]))<<"\t";
    }
    std::cerr<<std::endl;

#endif
    std::cerr<<"Rejecting this point!"<<std::endl;

#endif
    return false;
  }

    //4) re-write over Mmixing
    //Check first if the new basis is better
    //// Count number of zero entries for each
    size_t count0init=0,count0new=0;
    for(size_t i=0;i!=Phi.size();++i)
      for(size_t j=0;j!=Phi.size();++j){
    if(Mmixing[i][j]==0)
      ++count0init;
    if(d2VrebuiltEigen[(i+1)*Phi.size()+j]==0)
      ++count0new;
  }
    if(count0new>count0init){
#ifdef DEBUG
    std::clog<<std::endl;
    std::clog<<"Writing over original Mixing matrix which was: "<<std::endl;
    std::clog << endl << "Note: M_{jk} -> indices j (lines) are mass eigenstates and k (columns) are original states" << endl;

    PrintM1(Mmixing);
#endif
  //Massive states
  for(size_t i=0;i!=pos_new_basis.size();++i){
    for(size_t j=0; j!=Phi.size();++j)
      Mmixing[i][j]=d2VrebuiltEigen[(pos_new_basis[i]+1)*Phi.size()+j];
  }
  //Goldstones
  for(size_t i=pos_new_basis.size();i!=Phi.size();++i){
    for(size_t j=0; j!=Phi.size();++j)
      Mmixing[i][j]=d2VrebuiltEigen[(i-pos_new_basis.size()+1)*Phi.size()+j];
  }
  }
    else{
#ifdef VERBOSE
    std::clog<<std::endl<<"New basis is not better. Carrying on with original basis. "<<std::endl;
#endif
  }

  return true;
}

void Potential::a0treeIni(){

  vector< vector<int> > PowStates;
  PowStates.resize((Phi.size()*(Phi.size()+1))/2);
  for(size_t i=0; i!=(Phi.size()*(Phi.size()+1))/2;++i){
    PowStates[i].resize(Phi.size());
    for(size_t j=0; j!=Phi.size();++j){
      PowStates[i][j]=0;
      if(i<Phi.size() && i==j)
	PowStates[i][j]=2;
    }
  }

  size_t count=Phi.size();
  for(size_t i=0;i!=Phi.size();++i){
    for(size_t j=i+1;j<Phi.size();++j){
      PowStates[count][i]=1;
      PowStates[count][j]=1;
      ++count;
    }
  }

  // Now increment by the correct amount using all possible vertices

  for(size_t a=0; a!=Cp.size(); ++a){
    for(size_t nop=0;nop!=Mon[a].size();++nop){
      for(size_t i=0; i!=(Phi.size()*(Phi.size()+1))/2;++i){
	for(size_t j=0;j!=(Phi.size()*(Phi.size()+1))/2;++j){
	  double Prefac=1;
	  if(i<Phi.size())
	    Prefac/=sqrt(2);
	  if(j<Phi.size())
	    Prefac/=sqrt(2);
	  //test if this vertex contributes by checking powers
	  for(size_t k=0;k!=Phi.size();++k){
	    int p=PowStates[i][k]+PowStates[j][k];
	    /////////////
	    if(Mon[a][nop][k]==p)
	      Prefac*=factorial(p);
	    else{
	      Prefac=0;
	      break;
	    }
	  }
	  a0tree[i][j][a]+=Prefac*Mon[a][nop].coeff_get()/16e0/acos(-1);
	}
      }
    }
  }

#ifdef DEBUG
  /////Check: Print out matrix
  std::clog << std::endl << "****** Printing out Matrices used later for tree level unitarity check ********";
  printa0tree();
#endif

}

bool Potential::CheckTreeUni(){
  //Compute the matrix of amplitudes for the current parameter space point
  size_t dim=(Phi.size()*(Phi.size()+1))/2;
  for(size_t i=0; i<dim;++i){
    for(size_t j=0;j<dim;++j){
      a0treeSum[dim*i+j]=0;
      for(size_t a=0; a!=Cp.size(); ++a){
	a0treeSum[dim*i+j]+=a0tree[i][j][a]*Cp.L[a];
      }
    }
  }

  //Compute Eigenvalues (matrix is destroyed!)
  EigenSystemRsymNoSort(dim,a0treeSum,a0treeSum);

  for(size_t i=0;i<dim;++i){
    if(abs(a0treeSum[i])>0.5){
#ifdef VERBOSE
    std::clog<<endl<<"---> ScannerS: Point rejected by Potential::CheckTreeUni()."<<endl;
#endif
      return false;
    }
  }
  return true;
}

void Potential::BoundTreeUni(){
  //Compute the matrix of amplitudes for the current parameter space point
  size_t dim=(Phi.size()*(Phi.size()+1))/2;
  for(size_t i=0; i<dim;++i){
    for(size_t j=0;j<dim;++j){
      for(size_t a=0; a!=Cp.size(); ++a){
	if(abs(a0tree[i][j][a]*Cp.L.vmin(a))>0.5){
#ifdef VERBOSE
	  std::clog<< "Reseting Lmin[" << a <<"] = "<< Cp.L.vmin(a) <<" based on tree level unitarity to Lmin[" << a <<"] = ";
#endif
	  Cp.L.vmin(a)/=abs(2*Cp.L.vmin(a)*a0tree[i][j][a]);
#ifdef VERBOSE
	  std::clog<< Cp.L.vmin(a)<< endl;
#endif
	}
	if(abs(a0tree[i][j][a]*Cp.L.vmax(a))>0.5){

#ifdef VERBOSE
	  std::clog<< "Reseting Lmax[" << a <<"] = "<< Cp.L.vmax(a) <<" based on tree level unitarity to Lmax[" << a <<"] = ";
#endif
	  Cp.L.vmax(a)/=abs(2*Cp.L.vmax(a)*a0tree[i][j][a]);
#ifdef VERBOSE
	  std::clog<< Cp.L.vmax(a)<< endl;
#endif
	}
      }
    }
  }
}

bool Potential::CheckBounds()const{
  for(size_t a=0;a!=Cp.size();++a)
    if(Cp.L[a]<Cp.L.vgetmin(a) || Cp.L[a]>Cp.L.vgetmax(a)){
#ifdef VERBOSE
    std::clog<<endl<<"---> ScannerS: Point rejected by Potential::CheckBounds()."<<endl;
#endif
      return false;
    }
  for(size_t i=0;i!=Cp.Mass.size();++i)
    if(Cp.Mass[i]<Cp.Mass.vgetmin(i) || Cp.Mass[i]>Cp.Mass.vgetmax(i)){
#ifdef VERBOSE
    std::clog<<endl<<"---> ScannerS: Point rejected by Potential::CheckBounds()."<<endl;
#endif
      return false;
    }

  return true;
}

void Potential::PrintHeader() const{
  std::clog<<endl<<"***********************************************"<<endl;
  std::clog<<"*********** "<<scanners_version<<" ************"<<endl;
  std::clog<<"***********************************************" <<endl;
  std::clog<<endl<<"Potential Type: Mathematica Generated"<<endl;
  std::clog<<endl<<"---------- SUMMARY INFO FOR THIS RUN ---------- "<<endl;
  std::clog<<endl<< "The expanded potential is:"<<endl;
  std::clog<<endl<<(*this)<<endl;
  std::clog<<endl<<"The ranges of parameters to be scanned are:"<<endl;
  std::clog<<endl<<"Ranges for vacuum expectation values:"<<endl;
  for(size_t i=0;i!=Phi.size();++i)
    std::clog<< "phi"<< i<< " in ["<<Phi.vgetmin(i)<<":"<<Phi.vgetmax(i)<<"]"<< endl;
  std::clog <<endl<<"Ranges for couplings:"<< endl;
  for(size_t i=0;i!=Cp.L.size();++i)
    std::clog<<"L"<<i<<" in ["<<Cp.L.vgetmin(i)<<":"<<Cp.L.vgetmax(i)<<"]"<< endl;
  std::clog <<endl<<"Ranges for physical state masses:"<< endl;
  for(size_t i=0;i!=Cp.Mass.size();++i)
    std::clog<<"m"<<i<<" in ["<<Cp.Mass.vgetmin(i)<<":"<<Cp.Mass.vgetmax(i)<<"]"<<endl;
  std::clog<<endl;
  std::clog<<"The seed you are using for the random number generator is: "<<r.getseed()<< endl<< endl;
}


//////////////////////////////////////////////////////////////////////////
////////////////// VERBOSE MODE ROUTINES /////////////////////////////////
//////////////////////////////////////////////////////////////////////////

void Potential::printdV(){
  std::clog<< endl;
  for(std::vector< std::vector<double> >::iterator iter=dV.begin();iter!=dV.end();++iter){
    std::clog<< "| ";
    for(std::vector<double>::iterator ater=(*iter).begin();ater!=(*iter).end();++ater){
      std::clog<<scientific<<*ater<<"\t";
    }
    std::clog<<" |"<<endl;
  }
  std::clog<< endl;
}

void Potential::printd2V(){
  for(size_type a=0; a!=d2V[0][0].size();++a){
    std::clog<< " a = " <<a<<endl;
    for(size_type i=0;i!=d2V.size();++i){
      std::clog<< "| ";
      for(size_type j=0;j!=d2V.size();++j){
	std::clog<<scientific<<d2V[i][j][a]<<"\t";
      }
      std::clog<<" |"<<endl;
    }
    std::clog<< endl;
  }
  std::clog<< endl;
}

void Potential::printa0tree(){
  for(size_type a=0; a!=a0tree[0][0].size();++a){
    std::clog<< " a = " <<a<<endl;
    for(size_type i=0;i!=a0tree.size();++i){
      std::clog<< "| ";
      for(size_type j=0;j!=a0tree.size();++j){
	std::clog<<scientific<<a0tree[i][j][a]<<"\t";
      }
      std::clog<<" |"<<endl;
    }
    std::clog<< endl;
  }
  std::clog<< endl;
}

std::string Potential::name()const{
//returns the name of the type of potential used.
  string name("Mathematica generated");
  return name;
}
